/* Nacita ze vstupniho souboru shodne body a informace o rozmerech fotografii.
 * Kontroluje spravnost formatu vstupniho souboru.
 * Pri nacitani shodnych bodu eliminuje dvojice bodu, ktere jsou uvedeny vickrat
 * Autor: Beata Turonova
 */
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <gtk/gtk.h>
#include <glade/glade.h>
#include "input.h"
#include "points.h"
#include "surface.h"
#include "matrix.h"
#include "outputs.h"

int N;                                      //pocet shodnych bodu
double **p;                                 //matice s ulozenymi shodnymi body
double xmin_2D,ymin_2D,xmax_2D,ymax_2D;     //min a max hodnoty souradnic na fotografiich
int X3D;                                    //0-ne,1-ano
int WRL;                                    //0-ne,1-ano
int input_format2;                          //0-vlastni, 1=autopano
int model_type2;                            //0-projektivni, 1=metricky
char *data_file_name;                       //nazev souboru se vstupnimi daty
char *calib_file_name;                      //nazev souboru s kalibracnimi daty
double **K1t;                               //prvni kalibracni matice
double **K2t;                               //druha kalibracni matice

GladeXML *gxml;
GtkWidget *ar1;
GtkWidget *ar2;
GtkWidget *ar3;
GtkWidget *ar4;
GtkWidget *ok1;
GtkWidget *ok2;
GtkWidget *ok3;
GtkWidget *ok4;
GtkWidget *nn2;
GtkWidget *nn3;
GtkWidget *nn4;
GtkWidget *errorwin2;
GtkWidget *statuswin;
GtkWidget *run_ok1;
GtkWidget *run_ok2;


/**
* Nacte ze souboru cislo typu double
*
* @param f soubor, ze ktereho se cte
* @param m pro ulozeni nacteneho cisla
* @param err cislo chyby pro pripad neuspechu
* @return   0 - nacteni probehlo v poradku
*           1 - doslo k chybe
*/
int read_double(FILE *f,double* m,int err)
{
    if(fscanf(f,"%lf", m)!=1)
    {
        my_error(err);
        return 1;
    }
    else
        return 0;
}

/**
* Nacte ze souboru cislo typu int
*
* @param f soubor, ze ktereho se cte
* @param m pro ulozeni nacteneho cisla
* @param err cislo chyby pro pripad neuspechu
* @return   0 - nacteni probehlo v poradku
*           1 - doslo k chybe
*/
int read_int(FILE *f,int* m,int err)
{
    if(fscanf(f,"%d", m)!=1)
    {
        my_error(err);
        return 1;
    }
    else
        return 0;
}

/**
* Nacte ze vstupniho souboru souradnice bodu
*
* @param points vsutpni soubor
* @param c znak, za kterym zacina cislo
* @return point souradnice bodu
*/
double read_point(FILE* points, char c)
{
    char h;
    double point;

    h=getc(points);
    while (h!=c && h!=EOF)
        h=getc(points);

    if(read_double(points,&point,3)==1) return 1.0;

    return point;
}

/**
* Nacte ze vstupniho souboru rozmer fotografie
*
* @param points vsutpni soubor
* @param c znak, za kterym zacina cislo
* @return size rozmer fotografie
*/
int read_size(FILE* points, char c)
{
    char h;
    int size;

    h=getc(points);
    while (h!=c && h!=EOF)
        h=getc(points);

    if(h==EOF) {my_error(3);}
    else if(read_int(points,&size,3)==1) return 1;

    return size;
}

/**
* Vytvori z hodnot v souboru kalibracni matice
*
* @param K1 prvni kalibracni matice
* @param K2 druha kalibracni matice
* @return   0 - nacitani probehlo v poradku
*           1 - doslo k chybe
*/
int read_cal(double **K1, double **K2)
{
    int c;
    FILE* param; //soubor s kalibracnimi daty

    K1[2][2]=1.0;
    K2[2][2]=1.0;

    param=fopen(calib_file_name, "rt");
    if(!param)
    {
        my_error5(calib_file_name);
        return 1;
    }

    if(read_int(param,&c,4)==1) return 1;

    if (c==1)   //jsou hodnoty pouze k jedne kalibracni matici
    {
        if(read_double(param,&K1[0][0],4)==1) return 1;
        if(read_double(param,&K1[1][1],4)==1) return 1;
        if(read_double(param,&K1[0][1],4)==1) return 1;
        if(read_double(param,&K1[0][2],4)==1) return 1;
        if(read_double(param,&K1[1][2],4)==1) return 1;
        copy_matrix(K1,K2,3,3);
    }
    else //kalibracni matice jsou ruzne
    {
        if(read_double(param,&K1[0][0],4)==1) return 1;
        if(read_double(param,&K1[1][1],4)==1) return 1;
        if(read_double(param,&K1[0][1],4)==1) return 1;
        if(read_double(param,&K1[0][2],4)==1) return 1;
        if(read_double(param,&K1[1][2],4)==1) return 1;
        if(read_double(param,&K2[0][0],4)==1) return 1;
        if(read_double(param,&K2[1][1],4)==1) return 1;
        if(read_double(param,&K2[0][1],4)==1) return 1;
        if(read_double(param,&K2[0][2],4)==1) return 1;
        if(read_double(param,&K2[1][2],4)==1) return 1;
    }

    fclose(param);

    return 0;
}

/**
* Nacte ze vstupniho souboru typu autopano shodne body
*
* Behem nacitani odstranuje duplicitni body
*
* @param points vsutpni soubor
* @param p matice pro ulozeni shodnych bodu
* @param n pocet shodnych bodu
* @return c pocet jedinecnych shodnych bodu
*/
int read_autopano(FILE* points, double **p, int n)
{
    int i=0;
    int j;
    char h;
    double x1;  //pro ulozeni x-souradnice prvniho bodu
    int k;
    int c;      //skutecnu pocet shodnych bodu
    int m;
    int no;     //0-neni obsazen v matici, 1-je

    c=1;
    no=0;
    m=1;

    while (!feof(points))
    {
        if ((h=getc(points))=='#')
            i++;

        if (i==4)   //zacina nacitani souradnic
        {
            /*prvni bod*/
            p[0][0]=read_point(points,'x');
            p[0][1]=read_point(points,'y');
            p[0][2]=read_point(points,'X');
            p[0][3]=read_point(points,'Y');

            if(p[0][0]<p[0][2]){xmin_2D=p[0][0]; xmax_2D=p[0][2];}
            else {xmin_2D=p[0][2]; xmax_2D=p[0][0];}
            if(p[0][1]<p[0][3]){ymin_2D=p[0][1]; ymax_2D=p[0][3];}
            else {ymin_2D=p[0][3]; ymax_2D=p[0][1];}

            for (j=1;j<n;j++)
            {
                x1=read_point(points,'x');
                no=0;
                /*projde dosud ulozene body a zjisti, jestli tam souradnice uz neni*/
                for (k=0;k<m;k++)
                {
                    if (p[k][0]==x1)
                    {
                        no=1;
                        read_point(points,'y');
                        read_point(points,'X');
                        read_point(points,'Y');
                        break;
                    }
                }
                if (no==0) //bod dosud neni v matici obsazen
                {
                    p[m][0]=x1;
                    p[m][1]=read_point(points,'y');
                    p[m][2]=read_point(points,'X');
                    p[m][3]=read_point(points,'Y');

                    /*aktualizuje hranicni hodnoty*/
                    if(xmin_2D>p[m][0]) xmin_2D=p[m][0];
                    if(xmax_2D<p[m][0]) xmax_2D=p[m][0];
                    if(ymin_2D>p[m][1]) ymin_2D=p[m][1];
                    if(ymax_2D<p[m][1]) ymax_2D=p[m][1];
                    if(xmin_2D>p[m][2]) xmin_2D=p[m][2];
                    if(xmax_2D<p[m][2]) xmax_2D=p[m][2];
                    if(ymin_2D>p[m][3]) ymin_2D=p[m][3];
                    if(ymax_2D<p[m][3]) ymax_2D=p[m][3];
                    c++;
                    m++;
                }
            }
            break;
        }

    }

    return c;
}

/**
* Spusti namapovani textury
*
* Podle toho, jestli probehne namapovani textury
* bez problemu, zmeni vzhled statusoveho okna
*
* @param data gtk pointer na statuswin
*/
gboolean update_status3(gpointer data)
{
    int c;

    write_log(14);

    /* podle zvoleneho typu souboru vytvori vystupni soubor*/
    if(WRL==0 && X3D==0)
        c=map_texture(0, N);
	if(WRL==1)
        c=map_texture(0, N);
	if(X3D==1)
        c=map_texture(1, N);

    /*namapovani textury probehlo v poradku*/
    /*aktualizuje statusove okno*/
    if(c==0)
    {
        write_log(15);
        gtk_widget_hide(ar4);
        gtk_widget_show(ok4);
        gtk_widget_show(run_ok1);
        gtk_widget_show(run_ok2);
    }

    free_matrix(points3D);
	free_matrix(points2D);
	meshT.clear();
    return(FALSE);
}

/**
* Spusti vypocet rekonstrukce povrchu
*
* Podle toho, jestli probehla rekontrukce povrchu
* bez problemu, zmeni vzhled statusoveho okna
*
* @param data gtk pointer na statuswin
*/
gboolean update_status2(gpointer data)
{
    int c;

    write_log(12);
    c=mesh(points3D,N);

    /*rekonstrukce povrchu probehla v poradku*/
    /*aktualizuje statusove okno*/
    if(c==0)
    {
        write_log(13);
        gtk_widget_hide(ar3);
        gtk_widget_show(ok3);
        gtk_widget_hide(nn4);
        gtk_widget_show(ar4);
        g_timeout_add_seconds(1,update_status3,statuswin);
    }

    return(FALSE);
}

/**
* Zajisti nacteni kalibracnich
*
* Zkontroluje spravny format kalibracnich dat a nacte je.
* Spusti vypocet 3D souradnic.
* Podle toho, jestli kontrola dat, jejich nacteni
* a vypocet souradnic probehnou
* bez problemu, zmeni vzhled statusoveho okna
*
* @param data gtk pointer na statuswin
*/
gboolean update_status1(gpointer data)
{
    int i;
    int c;
    double avgx, avgy;  //prumerne hodnoty x-ovych a y-ovych souradnic
    int divide;         //delitel

    avgx=avgy=0;

    /*prumerne hodnoty x-ovych a y-ovych souradnic*/
    for (i=0; i<N; i++)
    {
        avgx=avgx+p[i][0]+p[i][2];
    }
    avgx=avgx/(2*N);

    if (model_type2==0)					//projektivni model
    {
        if(w1<=w2) divide=w2;
        else divide=w1;
        if(avgx>10)
        {
            /*upravi nactene souradnice*/
            for (i=0; i<N; i++)
            {
                p[i][0]=p[i][0]/divide;
                p[i][1]=p[i][1]/divide;
                p[i][2]=p[i][2]/divide;
                p[i][3]=p[i][3]/divide;
            }
            mult11=mult12=mult21=mult22=divide;
        }
        else
            mult11=mult12=mult21=mult22=1;

        write_log(9);
        write_log(10);
        c=compute3D(p,N,K1t,K2t,0);   //vypocet projektivnich 3D souradnic
    }
    else							//metricky model
    {
        if(avgx>10)
        {
            /*upravi nactene souradnice*/
            for (i=0;i<N;i++)
            {
                p[i][0]=p[i][0]/K1t[0][0];
                p[i][1]=p[i][1]/K1t[1][1];
                p[i][2]=p[i][2]/K2t[0][0];
                p[i][3]=p[i][3]/K2t[1][1];
            }
            mult11=K1t[0][0];
            mult12=K1t[1][1];
            mult21=K2t[0][0];
            mult22=K2t[1][1];
        }
        else
            mult11=mult12=mult21=mult22=1;

        write_log(9);
        write_log(10);
        c=compute3D(p,N,K1t,K2t,1);   //vypocet metrickych 3D souradnic
    }

    free_matrix(K1t);
    free_matrix(K2t);
    free_matrix(p);

    /*vypocet 3D souradnic probehl v poradku*/
    /*aktualizuje statusove okno*/
    if(c==0)
    {
        write_log(11);
        gtk_widget_hide(ar2);
        gtk_widget_show(ok2);
        gtk_widget_hide(nn3);
        gtk_widget_show(ar3);
        g_timeout_add_seconds(1,update_status2,statuswin);
    }

    return(FALSE);
}

/**
* Zajisti nacteni vstupnich dat
*
* Zkontroluje spravny format vstupnich dat a nacte je.
* Podle toho, jestli kontrola dat a jejich nacteni probehnou
* bez problemu, zmeni vzhled statusoveho okna
*
* @param name nazev vstupniho souboru
* @return   0 - vypocet probehl v poradku
*           1 - doslo k chybe
*/
int start_program(char *name)
{
    char h;
    int i;
    int n=0;        //pocet shondych bodu
    FILE* points;   //soubor se vstupnimi daty
    double x1;      //pro ulozeni x-ove souradnice prvniho bodu
    int no;         //0-bod dosud neni obsazen v matic, 1-je obsazen
    int k,m;
    int newn;       //spravny pocet neduplicitnich bodu

    write_log(1);
    write_log(8);

    i=0;
    m=1;
    newn=1;

    points=fopen(name, "rt");
    if(!points)
    {
        my_error5(name);
        return 1;
    }

    if (input_format2==0) //format vstupniho souboru je vlastni
    {
        /*nacte predpokaldany pocet shodnych bodu*/
        if(read_int(points,&n,3)==1) return 1;
        if (n<8)
        {
            my_error(2);
            return 1;
        }

        /*nacte rozmery fotografii*/
        if(read_int(points,&w1,3)==1) return 1;
        if(read_int(points,&h1,3)==1) return 1;
        if(read_int(points,&w2,3)==1) return 1;
        if(read_int(points,&h2,3)==1) return 1;

        p=matrix(n,4);

        /*nacte prvni dvojici shodnych bodu*/
        if(read_double(points,&p[0][0],3)==1) return 1;
        if(read_double(points,&p[0][1],3)==1) return 1;
        if(read_double(points,&p[0][2],3)==1) return 1;
        if(read_double(points,&p[0][3],3)==1) return 1;

        if(p[0][0]<p[0][2]){xmin_2D=p[0][0]; xmax_2D=p[0][2];}
        else {xmin_2D=p[0][2]; xmax_2D=p[0][0];}
        if(p[0][1]<p[0][3]){ymin_2D=p[0][1]; ymax_2D=p[0][3];}
        else {ymin_2D=p[0][3]; ymax_2D=p[0][1];}

        /*nacte shodne body*/
        for (i=1;i<n;i++)
        {
            if(read_double(points,&x1,3)==1) return 1;
            no=0;
            /*zkontroluje zda se dany bod uz v matici nevyskytuje*/
            for (k=0;k<m;k++)
            {
                if (p[k][0]==x1)
                {
                    no=1;
                    fscanf(points,"%lf %lf %lf");
                    break;
                }
            }
            if (no==0)
            {
                    p[m][0]=x1;
                    if(read_double(points,&p[m][1],3)==1) return 1;
                    if(read_double(points,&p[m][2],3)==1) return 1;
                    if(read_double(points,&p[m][3],3)==1) return 1;
                    if(xmin_2D>p[m][0]) xmin_2D=p[m][0];
                    if(xmax_2D<p[m][0]) xmax_2D=p[m][0];
                    if(ymin_2D>p[m][1]) ymin_2D=p[m][1];
                    if(ymax_2D<p[m][1]) ymax_2D=p[m][1];
                    if(xmin_2D>p[m][2]) xmin_2D=p[m][2];
                    if(xmax_2D<p[m][2]) xmax_2D=p[m][2];
                    if(ymin_2D>p[m][3]) ymin_2D=p[m][3];
                    if(ymax_2D<p[m][3]) ymax_2D=p[m][3];
                    newn++;
                    m++;
            }

        }//end for
        n=newn;
    }//end if
    else    //soubor je typu autopano
    {
        /*spocita pocet vstupnich bodu*/
        while (!feof(points))
        {
            if ((h=getc(points))=='\n')
                n++;

            /*nacte rozmery fotografii*/
            if (n==7)
            {
                if((w1=read_size(points,'w'))==1) return 1;
                if((h1=read_size(points,'h'))==1) return 1;
                while (h!='\n')
                    h=getc(points);
                if((w2=read_size(points,'w'))==1) return 1;
                if((h2=read_size(points,'h'))==1) return 1;
                n++;
            }
        }
        rewind(points);

        n=n-16;

        if (n<8)
        {
            my_error(2);
            return 1;
        }

        p=matrix(n,4);
        n=read_autopano(points,p,n);    //nacte shodne body
    }

    if (n<8)
    {
        my_error(2);
        return 1;
    }

    fclose(points);

    N=n;

    K1t=matrix(3,3);
    K2t=matrix(3,3);
    if (model_type2==1)
    {
        if (read_cal(K1t,K2t)==1)     //nacte kalibracni data
            return 1;
    }

    /*upravi statusove okno*/
    gtk_widget_hide(ar1);
    gtk_widget_show(ok1);
    gtk_widget_hide(nn2);
    gtk_widget_show(ar2);
    g_timeout_add_seconds(1,update_status1,statuswin);

    return 0;
}
