/* Nacita ze vstupniho souboru shodne body a informace o rozmerech fotografii.
 * Kontroluje spravnost formatu vstupniho souboru.
 * Pri nacitani shodnych bodu eliminuje dvojice bodu, ktere jsou uvedeny vickrat
 * Autor: Beata Turonova
 */
#ifndef _INPUT_H
#define _INPUT_H

extern int X3D;                 //0-ne,1-ano
extern int WRL;                 //0-ne,1-ano
extern int input_format2;       //0-vlastni, 1=autopano
extern int model_type2;         //0-projektivni, 1=metricky
extern char *data_file_name;    //nazev souboru se vstupnimi daty
extern char *calib_file_name;   //nazev souboru s kalibracnimi daty

extern double xmin_2D,ymin_2D,xmax_2D,ymax_2D;     //min a max hodnoty souradnic na fotografiich

extern GtkWidget *ar1;
extern GtkWidget *ar2;
extern GtkWidget *ar3;
extern GtkWidget *ar4;
extern GtkWidget *ok1;
extern GtkWidget *ok2;
extern GtkWidget *ok3;
extern GtkWidget *ok4;
extern GtkWidget *nn2;
extern GtkWidget *nn3;
extern GtkWidget *nn4;
extern GtkWidget *errorwin2;
extern GtkWidget *statuswin;
extern GtkWidget *run_ok1;
extern GtkWidget *run_ok2;

/**
* Nacte ze souboru cislo typu double
*
* @param f soubor, ze ktereho se cte
* @param m pro ulozeni nacteneho cisla
* @param err cislo chyby pro pripad neuspechu
* @return   0 - nacteni probehlo v poradku
*           1 - doslo k chybe
*/
int read_double(FILE *f,double* m,int err);

/**
* Nacte ze souboru cislo typu int
*
* @param f soubor, ze ktereho se cte
* @param m pro ulozeni nacteneho cisla
* @param err cislo chyby pro pripad neuspechu
* @return   0 - nacteni probehlo v poradku
*           1 - doslo k chybe
*/
int read_int(FILE *f,int* m,int err);

/**
* Nacte ze vstupniho souboru souradnice bodu
*
* @param points vsutpni soubor
* @param c znak, za kterym zacina cislo
* @return point souradnice bodu
*/
double read_point(FILE* points, char c);

/**
* Nacte ze vstupniho souboru rozmer fotografie
*
* @param points vsutpni soubor
* @param c znak, za kterym zacina cislo
* @return size rozmer fotografie
*/
int read_size(FILE* points, char c);

/**
* Vytvori z hodnot v souboru kalibracni matice
*
* @param K1 prvni kalibracni matice
* @param K2 druha kalibracni matice
* @return   0 - nacitani probehlo v poradku
*           1 - doslo k chybe
*/
int read_cal(double **K1, double **K2);

/**
* Nacte ze vstupniho souboru typu autopano shodne body
*
* Behem nacitani odstranuje duplicitni body
*
* @param points vsutpni soubor
* @param p matice pro ulozeni shodnych bodu
* @param n pocet shodnych bodu
* @return c pocet jedinecnych shodnych bodu
*/
int read_autopano(FILE* points, double **p, int n);

/**
* Spusti namapovani textury
*
* Podle toho, jestli probehne namapovani textury
* bez problemu, zmeni vzhled statusoveho okna
*
* @param data gtk pointer na statuswin
*/
gboolean update_status3(gpointer data);

/**
* Spusti vypocet rekonstrukce povrchu
*
* Podle toho, jestli probehla rekontrukce povrchu
* bez problemu, zmeni vzhled statusoveho okna
*
* @param data gtk pointer na statuswin
*/
gboolean update_status2(gpointer data);

/**
* Zajisti nacteni kalibracnich
*
* Zkontroluje spravny format kalibracnich dat a nacte je.
* Spusti vypocet 3D souradnic.
* Podle toho, jestli kontrola dat, jejich nacteni
* a vypocet souradnic probehnou
* bez problemu, zmeni vzhled statusoveho okna
*
* @param data gtk pointer na statuswin
*/
gboolean update_status1(gpointer data);

/**
* Zajisti nacteni vstupnich dat
*
* Zkontroluje spravny format vstupnich dat a nacte je.
* Podle toho, jestli kontrola dat a jejich nacteni probehnou
* bez problemu, zmeni vzhled statusoveho okna
*
* @param name nazev vstupniho souboru
* @return   0 - vypocet probehl v poradku
*           1 - doslo k chybe
*/
int start_program(char* points);

#endif
