/* Stara se o uzivatelske rozhrani
 * Autor: Beata Turonova
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <locale.h>
#include <gtk/gtk.h>
#include <glade/glade.h>
#include "input.h"
#include "outputs.h"

extern GladeXML  *gxml;
GtkWidget *cf;
GtkWidget *cl;
GtkWidget *okbtn;
GtkWidget *mainwin;
GtkWidget *aboutwin;

/**
* Nastavi hodnotu X3D
*
* @param widget x3d
*/
void set_type_x3d(GtkWidget *widget)
{
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
        X3D=1;
    else
        X3D=0;
}

/**
* Nastavi hodnotu WRL
*
* @param widget wrl
*/
void set_type_wrl( GtkWidget *widget)
{
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
        WRL=1;
    else
        WRL=0;
}

/**
* Nastavi typ modelu na metricky nebo projektivni
*
* @param widget
*/
void set_model_proj(GtkWidget *widget)
{
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))  //projektivni
    {
        model_type2=0;
        gtk_widget_set_sensitive(cl,FALSE);
        gtk_widget_set_sensitive(cf,FALSE);
    }
    else                                                            //metricky
    {
        model_type2=1;
        gtk_widget_set_sensitive(cl,TRUE);
        gtk_widget_set_sensitive(cf,TRUE);
    }
}

/**
* Nastavi vstupni format souboru
*
* @param widget
*/
void set_format1( GtkWidget *widget)
{
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
        input_format2=1;    //autopano
    else
        input_format2=0;    //vlastni
}

/**
* Ulozi jmeno souboru se vstupnimi daty
*
* @param widget
*/
void data_ok_sel( GtkWidget *w )
{
    data_file_name=gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (w));
}

/**
* Ulozi jmeno souboru s prvni fotografii
*
* @param widget
*/
void img1_ok_sel( GtkWidget *w )
{
    img1_file_name=gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (w));
}

/**
* Ulozi jmeno souboru s druhou fotografii
*
* @param widget
*/
void img2_ok_sel( GtkWidget *w )
{
    img2_file_name=gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (w));
}

/**
* Zavre okno s chybovym hlasenim
*
* @param w widget
*/
void error_off( GtkWidget *w )
{
    gtk_widget_hide(errorwin2);
    gtk_widget_set_sensitive(mainwin,TRUE);
}

/**
* Zavre okno s informacema o aplikaci
*
* @param w widget
*/
void about_off( GtkWidget *w )
{
    gtk_widget_hide(aboutwin);
    gtk_widget_set_sensitive(mainwin,TRUE);
}

/**
* Ulozi jmeno souboru s kalibracnimi daty
*
* @param w widget
*/
void calib_ok_sel( GtkWidget *w )
{
    calib_file_name=gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (w));
}

/**
* Ulozi cestu k vyslednemu modelu
*
* @param w widget
*/
void choose_path( GtkWidget *w )
{
    output_path=gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER (w));
}

/**
* Ulozi jmeno souboru s vyslednym modelem
*
* @param w widget
*/
void out_name( GtkWidget *w )
{
    output_name=gtk_entry_get_text(GTK_ENTRY(w));
}

/**
* Ukonci aplikaci
*/
void run_ok()
{
    write_log(17);
    gtk_main_quit();
}

/**
* Otevre okno s informace o aplikaci
*/
void show_about()
{
    gtk_widget_set_sensitive(mainwin,FALSE);
    gtk_widget_show(aboutwin);
}

/**
* Ukonci program po chybe
*/
void run_error()
{
    write_log(19);
    gtk_main_quit();
}

/**
* Ukonci program ze statusoveho okna
*/
void status_destroy()
{
    write_log(17);
    gtk_main_quit();
}

/**
* Zavre okno s chybovym hlasenim o spatnem formatu
* vstupniho souboru
*
* @param w widget
*/
void shut_errorwin2(GtkWidget *w)
{
    gtk_widget_hide_on_delete(w);
    gtk_widget_set_sensitive(mainwin,TRUE);
}

/**
* Po zadani vsech parametru zahaji start vypoctu modelu
*
* @param data ukazatel na statuswin
*/
gboolean run_app(gpointer data)
{
    start_program(data_file_name);
    return(FALSE);
}

/**
* Zkontroluje, zda byly zadany vsechny parametry
* a v pripade, ze ano zavola run_app()
*
* @param w widget
*/
int start_app( GtkWidget *widget)
{
    setlocale(LC_NUMERIC,"C");  //nastavi typ cisel s desetinnou teckou
    if(model_type2==0)          //projektivni model
    {
        if (data_file_name==NULL || img1_file_name==NULL || img2_file_name==NULL)   //nebyly zadany vsechny povinne soubory
        {
           gtk_widget_set_sensitive(mainwin,FALSE);
           gtk_widget_show(errorwin2);
           write_log(7);
        }
        else
        {
            gtk_widget_hide(mainwin);
            gtk_widget_show(statuswin);
            g_timeout_add_seconds(1,run_app,statuswin);
        }
    }
    else    //metricky model
    {
        if (data_file_name==NULL || img1_file_name==NULL || img2_file_name==NULL || calib_file_name==NULL) //nebyly zadany vsechny soubory
        {
           gtk_widget_set_sensitive(mainwin,FALSE);
           gtk_widget_show(errorwin2);
        }
        else
        {
            gtk_widget_hide(mainwin);
            gtk_widget_show (statuswin);
            g_timeout_add_seconds(1,run_app,statuswin);
        }
    }

    return 0;
}

/**
* Realizace uzivatelskeho rozhrani
*
* Inicializuje globalni promenne. Svaze udalosti v
* XML souboru s funkcema. Ulozi nektere ovladaci prvky
* do promennych.
*/
int main(int argc, char **argv)
{
    GtkWidget   *out_dir;
    GtkWidget   *errbtn;

    X3D=0;
    WRL=1;
    model_type2=0;
    input_format2=1;
    output_name="3DModel";

    gtk_init (&argc, &argv);

    gxml = glade_xml_new ("SPhotos3d.glade", NULL, NULL);

    mainwin = glade_xml_get_widget (gxml, "main_win");
    cl = glade_xml_get_widget (gxml, "calib_label");
    cf = glade_xml_get_widget (gxml, "calib_file");
    okbtn = glade_xml_get_widget (gxml, "okbtn");
    errbtn = glade_xml_get_widget (gxml, "ok_error2");
    statuswin = glade_xml_get_widget (gxml, "status");
    errorwin = glade_xml_get_widget (gxml, "errorwin");
    errorwin2 = glade_xml_get_widget (gxml, "errorwin2");
    aboutwin = glade_xml_get_widget (gxml, "aboutwin");
    errorm1 = glade_xml_get_widget (gxml, "errorm1");
    errorm2 = glade_xml_get_widget (gxml, "errorm2");
    errorm3 = glade_xml_get_widget (gxml, "errorm3");
    errorm4 = glade_xml_get_widget (gxml, "errorm4");
    errorm5 = glade_xml_get_widget (gxml, "errorm5");
    errorm6 = glade_xml_get_widget (gxml, "errorm6");
    errorm7 = glade_xml_get_widget (gxml, "errorm7");
    errorf = glade_xml_get_widget (gxml, "errorf");
    run_ok1 = glade_xml_get_widget (gxml, "run_ok");
    out_dir = glade_xml_get_widget (gxml, "output_dir");
    run_ok2 = glade_xml_get_widget (gxml, "img5");
    ar1 = glade_xml_get_widget (gxml, "img1");
    ar2 = glade_xml_get_widget (gxml, "img2");
    ar3 = glade_xml_get_widget (gxml, "img3");
    ar4 = glade_xml_get_widget (gxml, "img4");
    ok1 = glade_xml_get_widget (gxml, "ar1");
    ok2 = glade_xml_get_widget (gxml, "ar2");
    ok3 = glade_xml_get_widget (gxml, "ar3");
    ok4 = glade_xml_get_widget (gxml, "ar4");
    nn2 = glade_xml_get_widget (gxml, "nn2");
    nn3 = glade_xml_get_widget (gxml, "nn3");
    nn4 = glade_xml_get_widget (gxml, "nn4");

    gtk_widget_set_sensitive(cl,FALSE);
    gtk_widget_set_sensitive(cf,FALSE);

    output_path=gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER (out_dir));

    glade_xml_signal_connect (gxml, "on_x3d_check_toggled", G_CALLBACK (set_type_x3d));
    glade_xml_signal_connect (gxml, "on_wrl_check_toggled", G_CALLBACK (set_type_wrl));
    glade_xml_signal_connect (gxml, "on_autopanoformat_toggled", G_CALLBACK (set_format1));
    glade_xml_signal_connect (gxml, "on_start_clicked", G_CALLBACK (start_app));
    glade_xml_signal_connect (gxml, "on_open_input_file_set", G_CALLBACK (data_ok_sel));
    glade_xml_signal_connect (gxml, "on_img1_open_file_set", G_CALLBACK (img1_ok_sel));
    glade_xml_signal_connect (gxml, "on_img2_open_file_set", G_CALLBACK (img2_ok_sel));
    glade_xml_signal_connect (gxml, "on_calib_file_file_set", G_CALLBACK (calib_ok_sel));
    glade_xml_signal_connect (gxml, "on_output_name_changed", G_CALLBACK (out_name));
    glade_xml_signal_connect (gxml, "on_output_dir_selection_changed", G_CALLBACK (choose_path));
    glade_xml_signal_connect (gxml, "on_model_project_toggled", G_CALLBACK (set_model_proj));
    glade_xml_signal_connect (gxml, "on_ok_error2_clicked", G_CALLBACK (error_off));
    glade_xml_signal_connect (gxml, "on_okbtn_clicked", G_CALLBACK (run_ok));
    glade_xml_signal_connect (gxml, "on_ok_error_clicked", G_CALLBACK (run_error));
    glade_xml_signal_connect (gxml, "destroy", G_CALLBACK (gtk_main_quit));
    glade_xml_signal_connect (gxml, "on_status_destroy", G_CALLBACK (status_destroy));
    glade_xml_signal_connect (gxml, "on_status_delete_event", G_CALLBACK (status_destroy));
    glade_xml_signal_connect (gxml, "on_errorwin_destroy", G_CALLBACK (run_error));
    glade_xml_signal_connect (gxml, "on_errorwin2_delete_event", G_CALLBACK (shut_errorwin2));
    glade_xml_signal_connect (gxml, "on_close_btn_clicked", G_CALLBACK (gtk_main_quit));
    glade_xml_signal_connect (gxml, "on_about_btn_clicked", G_CALLBACK (show_about));
    glade_xml_signal_connect (gxml, "on_close_about_clicked", G_CALLBACK (about_off));
    glade_xml_signal_connect (gxml, "on_aboutwin_delete_event", G_CALLBACK (shut_errorwin2));

    g_object_unref (G_OBJECT (gxml));

    gtk_widget_show(mainwin);
    gtk_main ();

    return 0;
}
