/* Rutiny pro praci s maticema a vektorz
 * Autor: Beata Turonova
 */
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <gtk/gtk.h>
#include "matrix.h"
#include "outputs.h"

/**
* Vytvori matici typu double a vyplni ji nulami
*
* @param nr pocet radku
* @param nc pocet sloupcu
* @return matice typu double o velikosti nr x nc
*/
double **matrix(int nr, int nc)
{
	int i,j;
	double **m;

	m=(double **) malloc(nr*sizeof(double*));
	if (!m)
	{
        my_error(6);
        return NULL;
	}

	m[0]=(double *) malloc(nr*nc*sizeof(double));
	if (!m[0])
    {
        my_error(6);
        return NULL;
	}


	for(i=1;i<=(nr-1);i++) m[i]=m[i-1]+nc;

	for(i=0;i<nr;i++)
	{
		for (j=0;j<nc;j++)
		{
			m[i][j]=0.0;
		}
	}
	return m;
}

/**
* Vytvori matici typu int a vyplni ji nulami
*
* @param nr pocet radku
* @param nc pocet sloupcu
* @return matice typu int o velikosti nr x nc
*/
int **intMatrix(int nr, int nc)
{
	int i,j;
	int **m;

	m=(int **) malloc(nr*sizeof(int*));
	if (!m)
    {
        my_error(6);
        return NULL;
	}


	m[0]=(int *) malloc(nr*nc*sizeof(int));
	if (!m[0])
	{
        my_error(6);
        return NULL;
	}

	for(i=1;i<=(nr-1);i++) m[i]=m[i-1]+nc;

	for(i=0;i<nr;i++)
	{
		for (j=0;j<nc;j++)
		{
			m[i][j]=0;
		}
	}
	return m;
}

/**
* Vytvori jednotkovou matici typu double
*
* @param nr pocet radku
* @param nc pocet sloupcu
* @return jednotkova matice typu double o velikosti nr x nc
*/
double **matrixUnit(int nr, int nc)
{
	int i,j;
	double **m;

	m=(double **) malloc(nr*sizeof(double*));
	if (!m)
	{
        my_error(6);
        return NULL;
	}

	m[0]=(double *) malloc(nr*nc*sizeof(double));
	if (!m[0])
	{
        my_error(6);
        return NULL;
	}

	for(i=1;i<=(nr-1);i++) m[i]=m[i-1]+nc;

	for(i=0;i<nr;i++)
	{
		for (j=0;j<nc;j++)
		{
			if(i==j)
				m[i][j]=1.0;
			else
				m[i][j]=0.0;
		}
	}
	return m;
}


/**
* Alokuje pamet pro vektor typu double
*
* @param n velikost vektoru
* @return vektor typu double o velikosti n
*/
double *myVector(int n)
{
	double *v;

	v=(double *)malloc(n*sizeof(double));
	if (!v)
	{
        my_error(6);
        return NULL;
	}

	return v;
}

/**
* Vytiskne matici na standardni vystup
*
* @param M matice urcena k tisku
* @param m pocet radku
* @param n pocet sloupcu
* @param caption poznamka k tistene matici
*/
void print_matrix(double **M, int m, int n, char *caption)
{
	int i,j;

	printf("\n\n%s\n", caption);

	for(i=0;i<m;i++)
	{
		for (j=0;j<n;j++) printf("%f ", M[i][j]);
		printf("\n");
	}
	printf("\n");
}

/**
* Vytiskne vektor na standardni vystup
*
* @param v matice urcena k tisku
* @param n velikost vektoru
* @param caption poznamka k tistene matici
*/
void print_vector(double *v, int n, char *caption)
{
	int i;

	printf("\n\n%s\n", caption);

	for(i=0;i<n;i++)
	{
		printf("%f ", v[i]);
	}
	printf("\n");
}


