/* Rutiny pro praci s maticema a vektorz
 * Autor: Beata Turonova
 */
#ifndef _MATRIX_H
#define _MATRIX_H


/* uvolni pamet vyhrazenou pro matici m*/
#define free_matrix(m)  \
{                       \
	free(m[0]);         \
	free(m);            \
}

/*transponuje matici m1 o velikosti m*n a ulozi ji jako matici mt*/
#define transpose(m1,mt,m,n)    \
{                               \
	int i,j;                    \
	for(i=0;i<m;i++)            \
	{                           \
		for (j=0;j<n;j++)       \
		{                       \
			mt[j][i]=m1[i][j];  \
		}                       \
	}                           \
}

/* zkopiruje matici m1 do matice m2*/
#define copy_matrix(m1,m2,m,n)      \
{                                   \
	int i,j;                        \
	for (i=0;i<m;i++)               \
	{                               \
		for(j=0;j<n;j++)            \
		{                           \
			m2[i][j]=m1[i][j];      \
		}                           \
	}                               \
}

/* vynasobi matici m1 s matici m2, vysledkem je matice p
   m je pocet radku matice m1; n je pocet sloupcu matice m1; n2 pocet sloupcu
   matice m2, vysledna mitce p je tedy velikosti m*n2 */
#define product(m1,m2,p,m,n,n2)                 \
{                                               \
	int i,j,k;                                  \
                                                \
	for (i=0; i<m; i++)                         \
	{                                           \
      for (j=0; j<n2; j++)                      \
      {                                         \
         p[i][j]=0;                             \
         for (k=0; k<n; k++)                    \
         {                                      \
			 p[i][j] += m1[i][k]*m2[k][j];      \
         }                                      \
      }                                         \
	}                                           \
}

/*vynasobi tri matice m1 x m2 x m3 a vyslednou matici ulozi do p;
* m-pocet radku m1, n-pocet sloupcu m1, n2-pocet sloupcu m2,
* n3-pocet sloupcu m3 */
#define product3(m1,m2,m3,p,m,n,n2,n3)  \
{                                       \
	double **c1;                        \
	c1=matrix(m,n2);                    \
                                        \
	product(m1,m2,c1,m,n,n2);           \
	product(c1,m3,p,m,n2,n3);           \
                                        \
	free_matrix(c1);                    \
}


/**
* Vytvori matici typu double a vyplni ji nulami
*
* @param nr pocet radku
* @param nc pocet sloupcu
* @return matice typu double o velikosti nr x nc
*/
double **matrix(int nr, int nc);

/**
* Vytvori matici typu int a vyplni ji nulami
*
* @param nr pocet radku
* @param nc pocet sloupcu
* @return matice typu int o velikosti nr x nc
*/
int **intMatrix(int nr, int nc);

/**
* Vytvori jednotkovou matici typu double
*
* @param nr pocet radku
* @param nc pocet sloupcu
* @return jednotkova matice typu double o velikosti nr x nc
*/
double **matrixUnit(int nr, int nc);

/**
* Alokuje pamet pro vektor typu double
*
* @param n velikost vektoru
* @return vektor typu double o velikosti n
*/
double *myVector(int n);

/**
* Vytiskne matici na standardni vystup
*
* @param M matice urcena k tisku
* @param m pocet radku
* @param n pocet sloupcu
* @param caption poznamka k tistene matici
*/
void print_matrix(double **M, int m, int n, char *caption);

/**
* Vytiskne vektor na standardni vystup
*
* @param v matice urcena k tisku
* @param n velikost vektoru
* @param caption poznamka k tistene matici
*/
void print_vector(double *v, int n, char *caption);


#endif

