/* Vytvari vystupni soubory
 * Autor: Beata Turonova
 */
#include <math.h>
#include <list>
#include <string.h>
#include <time.h>
#include <gtk/gtk.h>
#include <stdio.h>
#include "outputs.h"
#include "surface.h"
#include "matrix.h"
#include "points.h"
#include "input.h"


double mult11, mult12;      //uprava souradnic na prvni fotografii
double mult21, mult22;      //uprava souradnic na druhe fotografii
int w1,h1,w2,h2;            //sirky a vysky prvni a druhe fotografie

char *img1_file_name;       //nazev prvni fotografie
char *img2_file_name;       //nazev druhe fotografie
char *output_path;          //umisteni vysledneho modelu
const char *output_name;    //nazev vysledneho modelu

GtkWidget *errorwin;
GtkWidget *errorf;
GtkWidget *errorm1;
GtkWidget *errorm2;
GtkWidget *errorm3;
GtkWidget *errorm4;
GtkWidget *errorm5;
GtkWidget *errorm6;
GtkWidget *errorm7;

/**
*Schova chybove hlasky
*/
void hide_messages()
{
    gtk_widget_hide(errorm2);
    gtk_widget_hide(errorm3);
    gtk_widget_hide(errorm4);
    gtk_widget_hide(errorm5);
    gtk_widget_hide(errorm6);
    gtk_widget_hide(errorm7);
    gtk_widget_hide(errorf);
}

/**
* Vypisuje chybova hlaseni na vystup
* i do log.txt
*
* @param ern cislo chyby
*/
void my_error(int ern)
{
    hide_messages();
    gtk_widget_show(errorwin);
    switch(ern)
    {
        case 2: gtk_widget_show(errorm2); write_log(2); break;
        case 3: gtk_widget_show(errorm3); write_log(3); break;
        case 4: gtk_widget_show(errorm4); write_log(4); break;
        case 6: gtk_widget_show(errorm6); write_log(6); break;
        case 7: gtk_widget_show(errorm7); write_log(22); break;
    }
}

/**
* Vypise, ktery soubor se nepodarilo
* otevrit
*
* @param f nazev souboru
*/
void my_error5(char *f)
{
    hide_messages();
    gtk_widget_show(errorwin);
    gtk_widget_show(errorm5);
    gtk_label_set_text(GTK_LABEL(errorf),f);
    gtk_widget_show(errorf);
    write_log(5);
}

/**
* Vytvori vystupni soubor s bodovym 3D modelem
*
* @param p3d matice s 3D souradnicema
* @param n pocet bodu
* @param typ vystupniho souboru: 0-wrl;1-x3d
*/
void write_points(double **p3d,int n,int type)
{
    int len;        //delka nazvu cesty a vystupniho souboru
    int i;
    FILE* p_out;    //vystupni soubor

    len=strlen(output_name);
	if(output_path!=NULL)
        len=len+strlen(output_path);

	char p_name[len+12];    //nazev vystupniho souboru vcetne cesty

	/*ulozi do p_name cestu,nazev souboru a spravnou priponu*/
	if(output_path!=NULL)
	{
        strcpy(p_name,output_path);
        if(strchr(output_path,'/')!=NULL)
            strcat(p_name,"/");
        else
            strcat(p_name,"\\");

        strcat(p_name,output_name);
	}
	else
	{
        strcpy(p_name,output_name);
	}

    if(type==0)
        {strcat(p_name,"_points.wrl");}
    else
        {strcat(p_name,"_points.x3d");}


    p_out=fopen(p_name,"wt");       //vytvori vystupni soubor
    if(!p_out){ write_log(20); return;}

    /*zapise vsechny nalezitosti do souboru*/
    if(type==0)
        fputs("#VRML V2.0 utf8\n\n", p_out);
    else
    {
        fputs("#X3D V3.0 utf8\n", p_out);
        fputs("PROFILE Interchange\n\n", p_out);
    }


	fprintf(p_out, "Viewpoint {\n\t description \"%s\" \n\t position", "3D body");
	fprintf(p_out," %f %f %f  \n }\n\n",cx, cy, fabs(2*(maxZ+cz)));
	fputs("Transform {\n", p_out);
	fputs("\t children Shape {\n", p_out);
	fputs("\t\t geometry PointSet {\n", p_out);
	fputs("\t\t\t coord Coordinate {\n", p_out);
	fputs("\t\t\t\t point [\n", p_out);

	/*vypise do souboru souradnice*/
	if(p3d[0][0]>0 && p3d[0][1]<0)
	{
        for(i=0;i<n;i++)
            fprintf(p_out,"\t\t\t\t %.20f %.20f %.20f ,\n",-p3d[i][0],-p3d[i][1],-p3d[i][2]);
	}
	else
	{
        for(i=0;i<n;i++)
            fprintf(p_out,"\t\t\t\t %.20f %.20f %.20f ,\n",p3d[i][0],p3d[i][1],p3d[i][2]);
	}

	fputs("\t\t\t\t ] \n\t\t\t }", p_out);
	fputs("\t\t\t\n\t } \n",p_out);
	fprintf(p_out, "\t }\n\t center %f %f %f \n", cx, cy, -cz);
	fputs("}\n",p_out);
	fclose(p_out);

	write_log(21);
}

/**
* Vytvori vystupni soubor log.txt
*
* @param ev druh hlaseni, ktere se ma vypsat
*/
void write_log(int ev)
{
    time_t t;           //cas zavolani funkce
    struct tm * tloc;   //mistni cas
    FILE* logfile;      //vystupni soubor
    int len;            //delka nazvu vystupniho souboru vcetne cesty

	if(output_path!=NULL)
        len=strlen(output_path);

	char log_name[len+8]; //nazev vystupniho souboru vcetne cesty

	/*ulozi do log_name cestu,nazev souboru a spravnou priponu*/
	if(output_path!=NULL)
	{
        strcpy(log_name,output_path);
        if(strchr(output_path,'/')!=NULL)
            strcat(log_name,"/");
        else
            strcat(log_name,"\\");

        strcat(log_name,"log.txt");
	}
	else
	{
        strcpy(log_name,"log.txt");
	}


    if(ev==1)   //prvni zapis - soubor je nutne vytvorit
    {
        logfile=fopen(log_name,"wt");
    }
    else        //soubor log.txt uz existuje
    {
        logfile=fopen(log_name,"at");
    }

    if(!logfile) return;

    time(&t);                               //zjisti aktualni cas
    tloc=localtime(&t);                     //premeni jej na mistni
    fprintf(logfile,"%s", asctime(tloc));   //vytiskne

    switch(ev)
    {
        case 0: fprintf(logfile,"\t - byl sputn program \n"); break;
        case 1: fprintf(logfile,"\t - byl zahjen vpoet 3D modelu \n"); break;
        case 2: fprintf(logfile,"\t - chyba: nedostaten poet shodnch bod => \n");
                fprintf(logfile,"\t\t => program mus bt ukonen \n"); break;
        case 3: fprintf(logfile,"\t - chyba: patn formt vstupnho souboru se shodnmi body => \n");
                fprintf(logfile,"\t\t => program mus bt ukonen \n"); break;
        case 4: fprintf(logfile,"\t - chyba: patn formt souboru s kalibranmi daty => \n");
                fprintf(logfile,"\t\t => program mus bt ukonen \n"); break;
        case 5: fprintf(logfile,"\t - chyba: nepodailo se otevt poadovan soubor => \n");
                fprintf(logfile,"\t\t => program mus bt ukonen \n"); break;
        case 6: fprintf(logfile,"\t - chyba: nepodailo se alokovat pam => \n");
                fprintf(logfile,"\t\t => program mus bt ukonen \n"); break;
        case 7: fprintf(logfile,"\t - chyba: nebyly zadny vechny vstupn soubory => \n");
                fprintf(logfile,"\t\t => nvrat zpt k vbru soubor \n"); break;
        case 8: fprintf(logfile,"\t - bylo zahjeno natn vstupnch dat \n"); break;
        case 9: fprintf(logfile,"\t - vstupn data spn natena \n"); break;
        case 10: fprintf(logfile,"\t - zahjen vpoet 3D souadnic \n"); break;
        case 11: fprintf(logfile,"\t - 3D souadnice byly spotny \n"); break;
        case 12: fprintf(logfile,"\t - zahjen vpoet povrchu \n"); break;
        case 13: fprintf(logfile,"\t - vpoet povrchu byl dokonen \n"); break;
        case 14: fprintf(logfile,"\t - zahjeno mapovn textury \n"); break;
        case 15: fprintf(logfile,"\t - mapovn textury ukoneno \n");
                 fprintf(logfile,"\t - vytvoen vstupn soubor s 3D modelem \n"); break;
        case 16: fprintf(logfile,"\t - program byl uivatelem ukonen \n"); break;
        case 17: fprintf(logfile,"\t - program byl ukonen \n"); break;
        case 19: fprintf(logfile,"\t - program byl po chyb ukonen \n"); break;
        case 20: fprintf(logfile,"\t - nepodarilo se vytvorit soubor pro zapis 3D bodu \n"); break;
        case 21: fprintf(logfile,"\t - byl vytvoren soubor s 3D body \n"); break;
        case 22: fprintf(logfile,"\t - chyba: patn data - vechny body jsou shodn => \n");
                 fprintf(logfile,"\t\t => program mus bt ukonen \n"); break;
    }

    fclose(logfile);
}

/**
* Vytvori vystupni soubor s vyslednym modelem
*
* @param ftype typ souboru: 0-wrl;1-x3d
* @param n pocet bodu
* @return   0 - vse probehlo v poradku
*           1 - doslo k chybe
*/
int map_texture(int ftype, int n)
{
	int len;                        //delka vystupniho souboru vcetne cesty
	int i;
	list<triangle>::iterator it;
    FILE *out;                      //vystupni soubor

    len=strlen(output_name);
	if(output_path!=NULL)
        len=len+strlen(output_path);

	char save_path[len+5];          //nazev vystupniho souboru vcetnce cesty
	if(output_path!=NULL)
	{
        strcpy(save_path,output_path);
        if(strchr(output_path,'/')!=NULL)
            strcat(save_path,"/");
        else
            strcat(save_path,"\\");

        strcat(save_path,output_name);
	}
	else
	{
        strcpy(save_path,output_name);
	}

    if(ftype==0)
        strcat(save_path,".wrl");
    else
        strcat(save_path,".x3d");

	/*vytvori vystupni soubor*/
	out=fopen(save_path, "wt");
	if(!out)
	{
	    my_error5(save_path);
        return 1;
	}

	/*vypise do souboru vsechny nalezitosti*/
	if(ftype==0)
        fputs("#VRML V2.0 utf8\n\n", out);
    else
    {
        fputs("#X3D V3.0 utf8\n", out);
        fputs("PROFILE Interchange\n\n", out);
    }

	fprintf(out, "Viewpoint {\n\t description \"%s\" \n\t position ", output_name);
	fprintf(out," %f %f %f  \n }\n\n",cx, cy, fabs(2*(maxZ+cz)));
	fputs("Transform {\n", out);
	fputs("\t children Shape {\n", out);
	fputs("\t\t geometry IndexedFaceSet {\n", out);
	fputs("\t\t\t coord Coordinate {\n", out);
	fputs("\t\t\t\t point [\n", out);

	/*vypise do souboru 3D souradnice*/
	if(points3D[0][0]>0 && points3D[0][1]<0)
	{
        for(i=0;i<n;i++)
            fprintf(out,"\t\t\t\t %.20f %.20f %.20f ,\n", -points3D[i][0],-points3D[i][1],-points3D[i][2]);
	}
	else
	{
        for(i=0;i<n;i++)
            fprintf(out,"\t\t\t\t %.20f %.20f %.20f ,\n", points3D[i][0],points3D[i][1],points3D[i][2]);
	}

    fputs("\t\t\t\t ] \n\t\t\t }", out);
	fputs("\n\t\t\t solid FALSE \n\t\t\t convex FALSE \n", out);
	fputs("\t\t\t texCoord TextureCoordinate {\n\t\t\t\t point [\n",out);

	/*vypise do souboru 3D texturovaci souradnice*/
	if(xmin_2D>0 && xmax_2D<1 && ymin_2D>0 && ymax_2D<1)
	{
	    for(i=0;i<n;i++)
            fprintf(out,"\t\t\t\t %.20f %.20f  ,\n",points2D[i][0],(0.5-points2D[i][1])+0.5);
	}
	else
	{
        for(i=0;i<n;i++)
            fprintf(out,"\t\t\t\t %.20f %.20f  ,\n",(points2D[i][0]*mult11)/w1,(0.5-((points2D[i][1]*mult12)/h1))+0.5);
	}

	fputs("\t\t\t\t ] \n\t\t\t }\n",out);

	fputs("\t\t\t coordIndex [\n", out);

	/*spoji body do trojuhelnikove site vytvorene v surface.cpp*/
	for(it=meshT.begin();it!=meshT.end();it++)
        fprintf(out, "\t\t\t\t %d %d %d %d -1 \n", (*it).ida, (*it).idb, (*it).idc, (*it).ida);

	fputs("\t\t\t ]\n\t\t\t texCoordIndex[\n",out);

	for(it=meshT.begin();it!=meshT.end();it++)
        fprintf(out, "\t\t\t\t %d %d %d %d -1 \n", (*it).ida, (*it).idb, (*it).idc, (*it).ida);

	fputs("\t\t\t ]\n",out);
	fputs("\t\t\t\n\t } \n",out);
	fputs("\t appearance Appearance{\n\t\t texture ImageTexture{\n",out);
	fprintf(out,"\t\t\t url \" %s \"\n\t\t\t }\n\t\t }\n ", img1_file_name);

    fprintf(out, "\t }\n\t center %f %f %f \n", cx, cy, -cz);

    fputs("}\n",out);

	fclose(out);


	return 0;
}
