/* Vypocet 3D souradnic projektivniho
 * nebo metrickeho modelu
 * Autor: Beata Turonova
 */
#include <math.h>
#include <gtk/gtk.h>
#include "surface.h"
#include "matrix.h"
#include "svdcmp.h"
#include "roots.h"
#include "outputs.h"
#include "input.h"

double **K1;            //prvni kalibracni matice
double **K2;            //druha kalibracni matice
double cx,cy,cz,maxZ;   //prumerne hodnoty 3D bodu a maximalni hodnota z souradnice

double **points2D;      //souradnice bodu 2D
double **points3D;	    //souradnice 3D bodu
int	n;                  //pocet shodnych bodu

/**
* Najde minimalni hodnotu obsazenou ve vektoru
*
* @param w nesetrideny vektor vlasntich cisel
* @param n velikost vektoru
* @return c index prvku s minimalni hodnotou ve vektoru w
*/
int find_min(double *w, int n)
{
	double min;
	int c,i;

	min=w[n-1];
	c=n-1;

	for(i=n-1; i>=0;i--)
	{
		if (w[i]<min)
		{
			min=w[i];
			c=i;
		}
	}
	return c;
}

/**
* Najde maximalni hodnotu obsazenou ve vektoru
*
* @param w nesetrideny vektor vlasntich cisel
* @param n velikost vektoru
* @return c index prvku s maximalni hodnotou ve vektoru w
*/
int find_max(double *w, int n)
{
	double max;
	int c,i;

	max=w[0];
	c=0;

	for(i=0; i<n;i++ )
	{
		if (w[i]>max)
		{
			max=w[i];
			c=i;
		}
	}

	return c;
}

/**
* Usporada rozklad na vlastni cisla
*
* Vektor vlastnich cisel bude setriden
* sestupne a podle toho budou prehozeny
* sloupce v maticich rozkladu.
* Vsechny matice jsou velikosti 3x3
* a vektory jsou velikosti 3!!!
*
* @param S puvodni prvni matice SVD rozkladu
* @param w nesetrideny vektor vlasntich cisel
* @param V puvodni druha matice SVD rozkladu
* @param U setridena matice S
* @param Vt setridena a transponovana matice V
* @param D setridena diagonalni matice s vlastnimi cisly
*/
void order_svd(double **S, double *w, double **V, double **U, double **Vt, double **D)
{

	int r,s,t;      //indexy vektoru
	double* wc;     //setrideny vektor
	double min,max; //min a mx hodnoty vektoru w
	int i;

	wc=myVector(3);
	min=w[2];
	t=2;
	max=w[0];
	r=0;

	/*najde min a max hodnotu vektoru w*/
	for(i=0;i<3;i++)
	{
		if (w[i]<min)
		{
			min=w[i];
			t=i;
		}
		if (w[i]>max)
		{
			max=w[i];
			r=i;
		}
	}
	if ((r+t)==2) s=1;
	else if ((r+t)==1) s=2;
	else s=0;

	/*Vztvori odpovidajici matice U a Vt*/
	for (i=0;i<3;i++)
	{
		U[i][0]=S[i][r];
		Vt[0][i]=V[i][r];
	}
	for (i=0;i<3;i++)
	{
		U[i][1]=S[i][s];
		Vt[1][i]=V[i][s];
	}
	for (i=0;i<3;i++)
	{
		U[i][2]=S[i][t];
		Vt[2][i]=V[i][t];
	}

	/*diagonalni D*/
	D[0][0]=w[r];
	D[1][1]=w[s];
	D[2][2]=w[t];

	for(i=0;i<3;i++)
		wc[i]=w[i];

	/*setridi vektor*/
	w[0]=wc[r];
	w[1]=wc[s];
	w[2]=wc[t];

	free(wc);
}

/**
* Provadi transformaci souradnic
*
* Presune je tak, abz jejich stred byl
* v pocatku souradnicoveho systemu.
* Zmeni jejich velikost tak, aby prumerna
* vzdalenost od pocatku byla rovna druhe
* odmocnine.
*
* @param fm matice souradnic
* @param T1 prvni transformacni matice
* @param T2 druha transformacni matice
*/
int Scaling(double **fm,double **T1, double **T2T)
{
	double tx[2];   //posun ve smeru osy x
	double ty[2];   //posun ve smeru osy y
	double s[2];    //zmena velikosti souradnic
	int i;
	double h;
	double **temp;  //matice pro docasne ulozeni souradnic

	tx[0] = 0;
	tx[1] = 0;
    ty[0] = 0;
	ty[1] = 0;
	h=1.0/n;

	temp=matrix(4,n);

	/*vypocet tx a ty*/
	for (i=0; i<n; i++)
	{
      tx[0] = tx[0] + fm[0][i];
      ty[0] = ty[0] + fm[1][i];
      tx[1] = tx[1] + fm[2][i];
      ty[1] = ty[1] + fm[3][i];
	}

	tx[0] = tx[0] * (h);
	ty[0] = ty[0] * (h);
	tx[1] = tx[1] * (h);
	ty[1] = ty[1] * (h);

    /*translace bodu tak, aby pocatek byl v 0,0 */
    for (i=0; i<n; i++)
	{
      temp[0][i] = fm[0][i] - tx[0];
      temp[1][i] = fm[1][i] - ty[0];
      temp[2][i] = fm[2][i] - tx[1];
      temp[3][i] = fm[3][i] - ty[1];
	}

	/*vypocet s*/
	s[0] = 0;
	s[1] = 0;

	for (i=0; i<n; i++)
	{
		s[0] = s[0] + sqrt(temp[0][i]*temp[0][i]+temp[1][i]*temp[1][i]);
		s[1] = s[1] + sqrt(temp[2][i]*temp[2][i]+temp[3][i]*temp[3][i]);
	}

	s[0] = s[0]* h;
	s[1] = s[1]* h;
	s[0] = sqrt(2.0) / s[0] ;
	s[1] = sqrt(2.0) / s[1] ;

    /*zmena velikosti souradnic tak, aby prumerna vzdalenost
	 *od pocatku byla rovna odmocnine ze dvou*/
	for (i=0; i<n; i++)
	{
        fm[0][i] = (fm[0][i]- tx[0]) * s[0];
        fm[1][i] = (fm[1][i]- ty[0]) * s[0];
        fm[2][i] = (fm[2][i]-tx[1]) * s[1];
        fm[3][i] = (fm[3][i]-ty[1]) * s[1];
	}

	if ((s[0]==0) || (s[1]==0))
	{
	    my_error(7);
	    return 1;
	}

	/*sestaveni transformacnich matic*/
	T1[0][0]=T1[1][1]=s[0];
	T2T[0][0]=T2T[1][1]=s[1];
	T1[2][2]=T2T[2][2]=1;

	T1[0][2]=-s[0]*tx[0];
	T1[1][2]=-s[0]*ty[0];
	T2T[2][0]=-s[1]*tx[1];
	T2T[2][1]=-s[1]*ty[1];

	free_matrix(temp);

	return 0;
}

/**
* Upravi matici F tak, aby odpovidala Frobeniove norme
*
* @param Fr nenormalizovana esenncialni matice
* @param F esencialni matice po normalizaci
*/
void Frobenius(double **Fr, double **F)
{
	double **U;     //pomocne pro SVD rozklad
	double **D;     //pomocne pro SVD rozklad
	double **V;     //matice s vlastnimi vektory
	double **Vt;    //transponovana V
	double *w;      //vektor s vlastnimi cisly

	V=matrix(3,3);
	w=myVector(3);
	svdcmp(Fr,3,3,w,V);

	U=matrix(3,3);				//prvni matice SVD rozkladu
	Vt=matrix(3,3);				//transponovana matice V
	D=matrix(3,3);				//diagonalni matice - na diagonale dva nejvetsi prvky z w serazeny sestupne

	order_svd(Fr,w,V,U,Vt,D);	//sestupne setrideni hodnot vekotru w a podle toho i matice Fr a V

	D[2][2]=0.0;

	product3(U,D,Vt,F,3,3,3,3);

	/*uvolneni pameti*/
	free_matrix(U);
	free_matrix(D);
	free_matrix(V);
	free_matrix(Vt);
	free(w);
}


/**
* Upravi matici E tak, aby odpovidala Frobeniove norme
*
* @param Er nenormalizovana esenncialni matice
* @param E esencialni matice po normalizaci
*/
void FrobeniusEssential(double **Er, double **E)
{
	double **U;     //pomocne pro SVD rozklad
	double **D;     //pomocne pro SVD rozklad
	double **V;     //matice s vlastnimi vektory
	double **Vt;    //transponovana V
	double *w;      //vektor s vlastnimi cisly
	double r;       //nova hodnota vlastnich cisel

	V=matrix(3,3);
	w=myVector(3);
	svdcmp(Er,3,3,w,V);

	U=matrix(3,3);				//prvni matice SVD rozkladu
	Vt=matrix(3,3);				//transponovana matice V
	D=matrix(3,3);				//diagonalni matice - na diagonale dva nejvetsi prvky z w serazeny sestupne

	order_svd(Er,w,V,U,Vt,D);	//sestupne setrideni hodnot vekotru w a podle toho i matice Fr a V

	r=(D[0][0]+D[1][1])/2;
	D[0][0]=r;
	D[1][1]=r;
	D[2][2]=0.0;

	product3(U,D,Vt,E,3,3,3,3);

	/*uvolneni pameti*/
	free_matrix(U);
	free_matrix(D);
	free_matrix(V);
	free_matrix(Vt);
	free(w);
}

/**
* Spocita fundamentalni matici F
*
* Sestavi rovnici Af=0 a vyresi ji metodou
* nejmensich ctvercu.
*
* @param fm matice se shodnymi body
* @param F fundamentalni matice
*/
void Solving_Projective(double **fm, double **F)
{
	double **A;         //matice rovnice
	double x1,y1,x2,y2; //souradnice
	double **V;         //matice s vlastnimi vektory
	double *w;          //vektor s vlastnimi cisly
	double **Fi;        //fundamentalni matice pred Frobeniovou normou
    int i,j,k,c;

	A=matrix(n,9);
	V=matrix(9,9);
	w=myVector(9);

	/*sestaveni matice A pro rovnici Af=0*/
	for (i=0; i<n; i++)
	{
      x1 = fm[0][i];
      y1 = fm[1][i];
      x2 = fm[2][i];
      y2 = fm[3][i];

	  /*jeden radek matice A*/
	  A[i][0] = x1*x2;
      A[i][1] = x2*y1;
      A[i][2] = x2;
      A[i][3] = y2*x1;
      A[i][4] = y1*y2;
      A[i][5] = y2;
      A[i][6] = x1;
      A[i][7] = y1;
      A[i][8] = 1.0;
	}

	svdcmp(A,n, 9, w, V);           //rozklad na vlastni cisla
    c=find_min(w,9);				//najde index nejmensiho cisla vektoru w

	Fi=matrix(3,3);

	k=0;
	for (i=0;i<3;i++)				//ulozi hodnoty z c-teho sloupce matice V do Fi=nenormalizovana fundamentalni matice
	{
		for (j=0;j<3;j++)
		{
			Fi[i][j]=V[k][c];
			k++;
		}
	}

	Frobenius(Fi,F);	            //normalizace fundamentalni matice

	/*uvolneni pameti*/
	free_matrix(A);
	free_matrix(V);
	free(w);
	free_matrix(Fi);
}

/**
* Spocita esencialni matici E
*
* Nejdrive pomoci kalibracnich dat normalizuje
* 2D souradnice a pak metodou nejmensich ctvercu
* vzresi rovnici x2^T E x1=0.
* Pokud je control v compute3D() nula,
* neni tato funkce volana!
*
* @param fm matice se shodnymi body
* @param E esencialni matice
*/
void Solving_Metric(double **fm,double **E)
{
	double **A;					//matice rovnice
	double x1,y1,x2,y2;         //normalizovane souradnice
	double **V;                 //matice s vlastnimi vektory
	double *w;                  //vektor vlastnich cisel
	double **Fi;                //esencialni matice pred Frobeniovou normou
	double **Kin1;				//inverzni matice K1
	double **Kin2;				//inverzni matice K2
	double **hx1;				// matice pro homogenni souradnice bodu x1
	double **hx2;				// matice pro homogenni souradnice bodu x2
	double **Kx1, **Kx2;		// matice pro nove upravene body  x1 a x2
    int i,j,k,c;

	A=matrix(n,9);
	V=matrix(9,9);
	w=myVector(9);
	Kin1=matrix(3,3);
	Kin2=matrix(3,3);
	Fi=matrix(3,3);
	hx1=matrix(3,1);
	hx2=matrix(3,1);
	Kx1=matrix(3,1);
	Kx2=matrix(3,1);

	//vytvoreni matic inverznich k matici K1 a K2
	Kin1[0][0]=1/K1[0][0];
	Kin1[1][1]=1/K1[1][1];
	Kin1[2][2]=1.0;
	Kin1[1][2]=-K1[1][2]/K1[1][1];
	Kin1[0][1]=-K1[0][1]/(K1[0][0]*K1[1][1]);
	Kin1[0][2]=(-K1[0][2]+K1[1][2]*K1[0][1])/K1[0][0];

	Kin2[0][0]=1/K2[0][0];
	Kin2[1][1]=1/K2[1][1];
	Kin2[2][2]=1.0;
	Kin2[1][2]=-K2[1][2]/K2[1][1];
	Kin2[0][1]=-K2[0][1]/(K2[0][0]*K2[1][1]);
	Kin2[0][2]=(-K2[0][2]+K2[1][2]*K2[0][1])/K2[0][0];

	/*sestaveni matice A pro rovnici Af=0*/
	for (i=0; i<n; i++)
	{
      hx1[0][0] = fm[0][i];
	  hx1[1][0] = fm[1][i];
	  hx1[2][0] = 1.0;

      hx2[0][0] = fm[2][i];
	  hx2[1][0] = fm[3][i];
	  hx2[2][0] = 1.0;

	  /*normalizace souradnic kalibracnimi daty*/
	  product(Kin1,hx1,Kx1,3,3,1);
	  product(Kin2,hx2,Kx2,3,3,1);

	  x1 = Kx1[0][0];
      y1 = Kx1[1][0];
      x2 = Kx2[0][0];
      y2 = Kx2[1][0];

	  /*jeden radek matice A*/
	  A[i][0] = x1*x2;
      A[i][1] = x2*y1;
      A[i][2] = x2;
      A[i][3] = y2*x1;
      A[i][4] = y1*y2;
      A[i][5] = y2;
      A[i][6] = x1;
      A[i][7] = y1;
      A[i][8] = 1.0;
	}

	svdcmp(A, n, 9, w, V);      //rozklad na vlastni cisla
	c=find_min(w,9);            //nalezeni indexu s nejmensim vlastnim cislem

	Fi=matrix(3,3);

	k=0;
	for (i=0;i<3;i++)			//ulozi hodnoty z c-teho sloupce matice V do Fi=nenormalizovana esencialni matice
	{
		for (j=0;j<3;j++)
		{
			Fi[i][j]=V[k][c];
			k++;
		}
	}


	FrobeniusEssential(Fi,E);   //normalizace essential matice podle Frobeniovy normy

	/*uvolneni pameti*/
	free_matrix(A);
	free_matrix(V);
	free(w);
    free_matrix(Fi);
    free_matrix(Kin1);
    free_matrix(Kin2);
    free_matrix(hx1);
    free_matrix(hx2);
    free_matrix(Kx1);
    free_matrix(Kx2);
}

/**
* Spocita matici P2 pro projektivni model
*
* @param F fundamentalni matice
* @param P2 pro ulozeni druhe matice fotoaparatu
*/
void projectiveP2(double **F, double **P2)
{
	double **ft, **fm;  //pomocne fundamentalni matice
	double **e2;        //druhy epipol
	double *w;          //vektor vlastnich cisel
	double **Ex;        //antiszmetricka matice odpovidajici e2
	int c;
	int i;

	//spocita epipolarni souradnici e2
	fm=matrix(3,3);
	w=myVector(3);
	ft=matrix(3,3);
	transpose(F,ft,3,3);

	svdcmp(ft,3,3,w,fm);

	c=find_min(w,3);

	e2=matrix(3,1);
	e2[0][0]=fm[0][c];
	e2[1][0]=fm[1][c];
	e2[2][0]=fm[2][c];

	//vyvtori antisymetrickou matici z e2
	Ex=matrix(3,3);

	Ex[0][1]=-e2[2][0];
	Ex[0][2]=e2[1][0];
	Ex[1][0]=e2[2][0];
	Ex[2][0]=-e2[1][0];
	Ex[2][1]=e2[0][0];
	Ex[1][2]=-e2[0][0];

	//vytvori matici P2=[Ex*F|e2]
	product(Ex,F,fm,3,3,3);
	copy_matrix(fm,P2,3,3);


	for(i=0;i<3;i++) P2[i][3]=e2[i][0];

	/*uvolneni pameti*/
	free_matrix(ft);
	free_matrix(fm);
	free_matrix(e2);
	free(w);
	free_matrix(Ex);
}

/**
* Urci, zda nalezena P2 vyhovuje
*
* Zkusi pomoci P2 vypocitat 3D souradnici
* nehomogenni metodou a na zaklade toho
* se rozhodne, zda P2 vyhovuje
*
* @param P2 pro ulozeni druhe matice fotoaparatu
* @return   0 - matice vyhovuje
*           1 - matice nevyhovuje
*/
int testP2(double **P2)
{
	int i, j;
	double **A, **U, **Ut, **P1;    //pomocne matice pro sestaveni rovnice
	double **V;                     //matice s vlastnimi vektory
	double *w;                      //vektor s vlastnimi cisly
	double **Xt;                    //ulozeni 3D souradnice
	double **b, **b2, **y;          //pomocne matice pro vypocet 3D souradnice

	A=matrix(4,4);
	U=matrix(4,3);
	Ut=matrix(3,4);
	Xt=matrix(3,1);
	b=matrix(4,1);
	b2=matrix(3,1);
	y=matrix(4,1);
	P1=matrixUnit(3,4);

    for (j=0;j<3;j++)
    {
        A[0][j]=points2D[0][0]*P1[2][j]-P1[0][j];
        A[1][j]=points2D[0][1]*P1[2][j]-P1[1][j];
        A[2][j]=points2D[0][2]*P2[2][j]-P2[0][j];
        A[3][j]=points2D[0][3]*P2[2][j]-P2[1][j];
    }

    b[0][0]=-(points2D[0][0]*P1[2][3]-P1[0][3]);
    b[1][0]=-(points2D[0][1]*P1[2][3]-P1[1][3]);
    b[2][0]=-(points2D[0][2]*P2[2][3]-P2[0][3]);
    b[3][0]=-(points2D[0][3]*P2[2][3]-P2[1][3]);

    copy_matrix(A,U,4,3);

    w=myVector(3);
    V=matrix(3,3);

    svdcmp(U,4,3,w,V);
    transpose(U,Ut,4,3);
    product(Ut,b,b2,3,4,1);

    for(i=0;i<3;i++)
        y[i][0]=b2[i][0]/w[i];

    product(V,y,Xt,3,3,1);

    /*uvolneni pameti*/
    free_matrix(A);
	free_matrix(U);
	free_matrix(Ut);
	free_matrix(P1);
	free_matrix(b);
	free_matrix(b2);
	free_matrix(y);
	free_matrix(V);
	free(w);

	if (Xt[0][0]>0)
	{
		free_matrix(Xt);
		return 0;   //uspech
	}
	else
	{
		free_matrix(Xt);
		return 1;  //neuspech
	}

}


/**
* Spocita matici P2 pro metricky model
*
* @param F fundamentalni matice
* @param P2 pro ulozeni druhe matice fotoaparatu
* @param control udava, zda je matice F fundamentali nebo esencialni
*/
void metricP2(double **F, double **P2, int control)
{
	double **E;				//essential matrix
	double **Kt;			//transponovana matice K
	double **U, **V;		//matice k SVD rozkladu
	double **Vt;            //transponovana V
	double **R1, **R2;		//rotacni matice
	double **W, **Zz;		//matice k vypoctu R1 a R2
	double **Wt;			//transponova W
	double *w;				//vektor vlastnich cisel
	double **D;				//diagonalni matice
	double **S, **Ut;		//kontrolni matice S
	int i, j;
	double *wc;             //serazeny vektor s vlastnimi cisly
	int c;
	double **test;          //test zda matice vyhovuje
    double diff1, diff2;    //rozdily v hodnotach prvku matic R1 a R2

	Kt=matrix(3,3);
	E=matrix(3,3);

	transpose(K2,Kt,3,3);

	if (control==0) {copy_matrix(F,E,3,3);}     //matice je esencialni
	else {product3(Kt,F,K1,E,3,3,3,3);}         //matice je fundamentalni a je treba ji vynasobit kalibracnimi daty

	free_matrix(Kt);

	w=myVector(3);
	wc=myVector(3);
	V=matrix(3,3);
	D=matrix(3,3);
	Vt=matrix(3,3);
	U=matrix(3,3);
	S=matrix(3,3);
	Ut=matrix(3,3);
	test=matrix(4,1);

	svdcmp(E,3,3,w,V);          //rozklad na vlastni cisla

	c=find_max(w,3);

	for(i=0;i<3;i++)
		wc[i]=w[i];

	w[0]=w[0]/wc[c];
	w[1]=w[1]/wc[c];
	w[2]=w[2]/wc[c];

	order_svd(E,w,V,U,Vt,D);    //serazeni roykladu sestupne podle vlastnich cisel

	transpose(Vt,V,3,3);

	free_matrix(V);

	W=matrix(3,3);
	Zz=matrix(3,3);

	W[0][1]=-1.0;
	W[1][0]=W[2][2]=1.0;

	Zz[0][1]=1.0;
	Zz[1][0]=-1.0;

	transpose(U,Ut,3,3);
	product3(U,Zz,Ut,S,3,3,3,3);

	for(i=0;i<3;i++)
		for(j=0;j<3;j++)
			U[i][j]=U[i][j]*wc[c];

    /*sestaveni matice R1 a R2*/
	R1=matrix(3,3);
    product3(U,W,Vt,R1,3,3,3,3);

    R2=matrix(3,3);
    Wt=matrix(3,3);

    transpose(W,Wt,3,3);
    product3(U,Wt,Vt,R2,3,3,3,3);

    /*urci, ktera matice se blizi matici antiszmetricke*/
    diff1=fabs(R1[0][1]+R1[1][0])+fabs(R1[0][2]+R1[2][0])+fabs(R1[1][2]+R1[2][1]);
    diff2=fabs(R2[0][1]+R2[1][0])+fabs(R2[0][2]+R2[2][0])+fabs(R2[1][2]+R2[2][1]);

	if(diff1<diff2)
	{
		copy_matrix(R1,P2,3,3);
	}
	else
	{
        copy_matrix(R2,P2,3,3);
	}

	for(i=0; i<3;i++)
			P2[i][3]=U[i][2];

	/*otestuje zda je vzber u spravny*/
	if (testP2(P2)!=0)
	{
		for(i=0; i<3;i++)
			P2[i][3]=-U[i][2];
	}

	/*uvolneni pameti*/
	free_matrix(E);
	free_matrix(U);
	free_matrix(Vt);
	free_matrix(R1);
	free_matrix(R2);
    free_matrix(Wt);
	free_matrix(W);
	free_matrix(Zz);
	free_matrix(D);
	free_matrix(S);
	free_matrix(Ut);
	free_matrix(test);
	free(w);
	free(wc);
}

/**
* Optimalizuje souradnice bodu
*
* Optimalizuje souradnice 2D bodu.
* Obsahuje hledani korenu polynomu
* stupne 6 - vyuziva funkci roots.
*
* @param p matice se shodnymi 2D body
* @param F fundamentalni matice
* @param opt matice pro ulozeni optimalizovanych bodu
*/
void optimal(double **p, double **F, double **opt)
{
	double **T1, **T2;                  //transformacni matice
	int i,j;
	double **Fn;                        //pomocna fundamentalni matice
	double **e1,**e2;                   //vektory - epipoly
	double **R1, **R2;                  //rotacni matice
	double **ft;                        //druha pomocna fundamentalni matice
	double **V;                         //matice s vlastnimi vektory
	double *w;                          //vektor s vlastnimi cisly
	double **R1t, **R2t;                //transponovane rotacni matice
	int min;
	double **cont1, **cont2, **cont3;   //matice pro kontrolni vypocty
	double s;                           //pomocna promenna pro normalizaci epipolu
	double f1,f2;                       //pro vypocet koeficientu polynomu
	double a,b,c,d;                     //pro vypocet koeficientu polynomu
	fcomplex t[7];                      //reprezentace polyomu
	fcomplex r[7];                      //nalezene koreny
	double cost;                        //hodnota funkce
	double mincost;                     //minimalni hodnota funkce
	double b0,b1,b2;                    //mezivypocty
	int index;                          //index korenu s nejmensi vahovou funkci
	double **x1, **x2;                  //normalizovane body
	double g;                           //hodnota polynomu
	double atb,ctd;                     //mezivypocty
	double f1_2,f1_3,f1_4;              //f1^2, f1^3, f1^4
	double f2_2,f2_3,f2_4;              //f2^2, f2^3, f2^4
	double a_2,a_3,a_4;                 //a^2, a^3, a^4
	double b_2,b_3,b_4;                 //b^2, b^3, b^4
	double c_2,c_3,c_4;                 //c^2, c^3, c^4
	double d_2,d_3,d_4;                 //d^2, d^3, d^4


	T1=matrixUnit(3,3);
	T2=matrixUnit(3,3);
	Fn=matrix(3,3);
	e1=matrix(3,1);
	e2=matrix(1,3);
	ft=matrix(3,3);
	w=myVector(3);
	V=matrix(3,3);
	R1=matrix(3,3);
	R2=matrix(3,3);
	R1t=matrix(3,3);
	R2t=matrix(3,3);
	x1=matrix(3,1);
	x2=matrix(3,1);

	cont1=matrix(3,1);
	cont2=matrix(3,1);
	cont3=matrix(1,1);

	for(i=0;i<n;i++)
	{
		/*transformacni matice*/
		T1[0][2]=p[i][0];
		T1[1][2]=p[i][1];
		T2[2][0]=p[i][2];
		T2[2][1]=p[i][3];

		product3(T2,F,T1,Fn,3,3,3,3);
		transpose(Fn,ft,3,3);
		svdcmp(ft,3,3,w,V);

		min=find_min(w,3);

		/*druhy epipol*/
		e2[0][0]=V[0][min];
		e2[0][1]=V[1][min];
		e2[0][2]=V[2][min];

		copy_matrix(Fn,ft,3,3);
		svdcmp(ft,3,3,w,V);
		min=find_min(w,3);

        /*prvni epipol*/
		e1[0][0]=V[0][min];
		e1[1][0]=V[1][min];
		e1[2][0]=V[2][min];

		s=sqrt(e1[0][0]*e1[0][0]+e1[1][0]*e1[1][0]);

		e1[0][0]=e1[0][0]/s;
		e1[1][0]=e1[1][0]/s;

		s=sqrt(e2[0][0]*e2[0][0]+e2[0][1]*e2[0][1]);

		e2[0][0]=e2[0][0]/s;
		e2[0][1]=e2[0][1]/s;

		/*sestaveni rotacnich matic*/
		R1[0][0]=e1[0][0];
		R1[0][1]=e1[1][0];
		R1[1][0]=-e1[1][0];
		R1[1][1]=e1[0][0];
		R1[2][2]=1.0;

		R2[0][0]=e2[0][0];
		R2[0][1]=e2[0][1];
		R2[1][0]=-e2[0][1];
		R2[1][1]=e2[0][0];
		R2[2][2]=1.0;

		transpose(R1,R1t,3,3);
		product3(R2,Fn,R1t,ft,3,3,3,3);

		/*pro vypocet hodnoty koeficientu polynomu*/
		f1=e1[2][0];
		f2=e2[0][2];
		a=ft[1][1];
		b=ft[1][2];
		c=ft[2][1];
		d=ft[2][2];

		f1_2=f1*f1;
		f1_3=f1*f1*f1;
		f1_4=f1*f1*f1*f1;

		f2_2=f2*f2;
		f2_3=f2*f2*f2;
		f2_4=f2*f2*f2*f2;

		a_2=a*a;
		a_3=a*a*a;
		a_4=a*a*a*a;

		b_2=b*b;
		b_3=b*b*b;
		b_4=b*b*b*b;

		c_2=c*c;
		c_3=c*c*c;
		c_4=c*c*c*c;

		d_2=d*d;
		d_3=d*d*d;
		d_4=d*d*d*d;

		/*zadani koeficientu polynomu*/
		t[0]=Complex(b_2*c*d-a*b*d_2,0);
		t[1]=Complex(2*b_2*d_2*f2_2+d_4*f2_4+b_4+b_2*c_2-a_2*d_2,0);
		t[2]=Complex(4*a*b_3 + 4*a*b*d_2*f2_2 + 4*b_2*c*d*f2_2 + 4*c*d_3*f2_4 - a_2*c*d
			+ a*b*c_2 - 2*a*b*d_2*f1_2 + 2*b_2*c*d*f1_2,0);
		t[3]=Complex(6*a_2*b_2 + 2*a_2*d_2*f2_2 + 8*a*b*c*d*f2_2 + 2*b_2*c_2*f2_2 + 6*c_2*d_2*f2_4
			- 2*a_2*d_2*f1_2+2*b_2*c_2*f1_2,0);
		t[4]=Complex(2*a*b*c_2*f1_2 - 2*a_2*c*d*f1_2 - a*b*d_2*f1_4 + b_2*c*d*f1_4 + 4*a_3*b
			+ 4*c_3*d*f2_4 + 4*a*b*c_2*f2_2 + 4*a_2*c*d*f2_2,0);
		t[5]=Complex(a_4 - a_2*d_2*f1_4 + b_2*c_2*f1_4 + f2_4*c_4 + 2*f2_2*a_2*c_2,0);
		t[6]=Complex(a*b*c_2*f1_4-a_2*c*d*f1_4,0);

		/*vypocet korenu polynomu*/
		zroots(t,6,r,1);

		/*nalezeni korene, pro ktery je hodnota polynomu minimalni*/
		b0=r[1].r*r[1].r;
		b1=(c*r[1].r+d)*(c*r[1].r+d);
		b2=(a*r[1].r+b)*(a*r[1].r+b);
		mincost=(b0/(1+f1*f1*b0)) + (b1/(b2+f2*f2*b1));
		index=1;
		for(j=2;j<7;j++)
		{
			b0=r[j].r*r[j].r;
			b1=(c*r[j].r+d)*(c*r[j].r+d);
			b2=(a*r[j].r+b)*(a*r[j].r+b);
			cost=(b0/(1+f1*f1*b0))+(b1/(b2+f2*f2*b1));
			if(cost<mincost)
			{
				mincost=cost;
				index=j;
			}
		}

        /*kontrola hodnoty funkce v nekonecnu*/
        if(f1!=0 && f2!=0)
        {
            cost=(1/f1_2) + (c_2/(a_2+f2_2*c_2));
            if(cost<mincost)
            {
                g=cost;
                index=-1;
            }
        }

        if(index!=-1)
            g=r[index].r;

		atb=(a*g+b);
		ctd=(c*g+d);

		/*normalizovane body*/
		x1[0][0]=g*g*f1;
		x1[1][0]=g;
		x1[2][0]=g*g*f1*f1+1;

		x2[0][0]=f2*ctd*ctd;
		x2[1][0]=-atb*ctd;
		x2[2][0]=f2*f2*ctd*ctd+atb*atb;

		T2[0][2]=T2[2][0];
		T2[1][2]=T2[2][1];
		T2[2][0]=0.0;
		T2[2][1]=0.0;
		transpose(R2,R2t,3,3);

		/*aplikovani transformaci na normalizovane body*/
		product3(T1,R1t,x1,cont1,3,3,3,1);
		product3(T2,R2t,x2,cont2,3,3,3,1);

		opt[i][0]=cont1[0][0]/cont1[2][0];
		opt[i][1]=cont1[1][0]/cont1[2][0];
		opt[i][2]=cont2[0][0]/cont2[2][0];
		opt[i][3]=cont2[1][0]/cont2[2][0];
	}

    /*uvolneni pameti*/
    free_matrix(T1);
    free_matrix(T2);
	free_matrix(Fn);
	free_matrix(e1);
	free_matrix(e2);
	free_matrix(R1);
	free_matrix(R2);
	free_matrix(ft);
	free_matrix(V);
	free_matrix(R1t);
	free_matrix(R2t);
	free_matrix(cont1);
	free_matrix(cont2);
	free_matrix(cont3);
	free_matrix(x1);
	free_matrix(x2);
	free(w);
}

/**
* Spocita 3D souradnice homogenni metodou
*
* Vyuziva rozkladu na vlastni cisla.
* Spocita hodnoty globalnich promennych cx,cy,cz
* a maxZ.
* Neni v tomto programu volana!
*
* @param P1 prvni matice fotoaparatu
* @param P2 druha matice fotoaparatu
* @param p matice se shodnymi 2D body
*/
void homogen_method(double **P1, double **P2, double **p)
{
	int i, j, k, c;
	double **A, **At, **AtA;    //pomocne matice pro sestaveni rovnic
	double **V;                 //a rozkladu na vlastni cisla
	double **Xm;                //pro docasne ulozeni 3D souradnic bodu
	double *w;                  //vektor pro ulozeni vlastnich cisel
	int first;

	A=matrix(4,4);
	At=matrix(4,4);
	AtA=matrix(4,4);
	Xm=matrix(n,4);

	cx=cy=cz=0.0;
    first=0;

	/*spocita 3D souradnice*/
	for (k=0;k<n;k++)
	{
		for (j=0;j<4;j++)
		{
			A[0][j]=p[k][0]*P1[2][j]-P1[0][j];
			A[1][j]=p[k][1]*P1[2][j]-P1[1][j];
			A[2][j]=p[k][2]*P2[2][j]-P2[0][j];
			A[3][j]=p[k][3]*P2[2][j]-P2[1][j];
		}

		transpose(A,At,4,4);
		product(At,A,AtA,4,4,4);

		w=myVector(4);
		V=matrix(4,4);

		svdcmp(AtA,4,4,w,V);

		c=find_min(w,4);

		for (i=0;i<4;i++)
		{
			Xm[k][i]=V[i][c];
		}

		cx=cx+Xm[k][0]/Xm[k][3];
		cy=cy+Xm[k][1]/Xm[k][3];
		cz=cz+Xm[k][2]/Xm[k][3];

		points3D[k][0]=Xm[k][0]/Xm[k][3];
		points3D[k][1]=Xm[k][1]/Xm[k][3];
		points3D[k][2]=Xm[k][2]/Xm[k][3];

		if(first==0)
        {
            maxZ=points3D[k][2];
            first=1;
        }
        else
        {
            if (maxZ<points3D[k][2]) maxZ=points3D[k][2];
        }

	}

	cx=cx/n;
	cy=cy/n;
	cz=cz/n;

    /*uvolneni pameti*/
	free_matrix(A);
	free_matrix(At);
	free_matrix(AtA);
	free_matrix(Xm);
	free_matrix(V);
	free(w);
}

/**
* Spocita 3D souradnice nehomogenni metodou
*
* Vyuziva rozkladu na vlastni cisla.
* Spocita hodnoty globalnich promennych cx,cy,cz
* a maxZ.
*
* @param P1 prvni matice fotoaparatu
* @param P2 druha matice fotoaparatu
* @param p matice se shodnymi 2D body
*/
void inhomogen_method(double **P1, double **P2, double **p)
{
	int i, j, k;
	double **A, **U, **Ut;      //pomocne matice pro sestaveni rovnic
	double **V;                 //a rozkladu na vlastni cisla
	double *w;                  //vektor pro ulozeni vlastnich cisel
	double **Xt;                //pro docasne ulozeni 3D souradnic bodu
	double **b, **b2, **y;      //pomocne vektory k ulozeni vysledku
    int first;                  //0-prvni pruchod,1-druhy pruchod

	A=matrix(4,4);
	U=matrix(4,3);
	Ut=matrix(3,4);
	Xt=matrix(3,1);
	points3D=matrix(n,3);
	b=matrix(4,1);
	b2=matrix(3,1);
	y=matrix(4,1);

	cx=cy=cz=0.0;
	first=0;

    /*spocita 3D souradnice*/
	for (k=0;k<n;k++)
	{
		for (j=0;j<3;j++)
		{
			A[0][j]=p[k][0]*P1[2][j]-P1[0][j];
			A[1][j]=p[k][1]*P1[2][j]-P1[1][j];
			A[2][j]=p[k][2]*P2[2][j]-P2[0][j];
			A[3][j]=p[k][3]*P2[2][j]-P2[1][j];
		}

		b[0][0]=-(p[k][0]*P1[2][3]-P1[0][3]);
		b[1][0]=-(p[k][1]*P1[2][3]-P1[1][3]);
		b[2][0]=-(p[k][2]*P2[2][3]-P2[0][3]);
		b[3][0]=-(p[k][3]*P2[2][3]-P2[1][3]);

		copy_matrix(A,U,4,3);

		w=myVector(3);
		V=matrix(3,3);

		svdcmp(U,4,3,w,V);
		transpose(U,Ut,4,3);
		product(Ut,b,b2,3,4,1);

		for(i=0;i<3;i++)
			y[i][0]=b2[i][0]/w[i];

		product(V,y,Xt,3,3,1);

		cx=cx+Xt[0][0];
		cy=cy+Xt[1][0];
		cz=cz+Xt[2][0];

		points3D[k][0]=Xt[0][0];
		points3D[k][1]=Xt[1][0];
		points3D[k][2]=Xt[2][0];

        /*nalezne maximalni hodnotu souradince z*/
        if(first==0)
        {
            maxZ=Xt[2][0];
            first=1;
        }
        else
        {
            if (maxZ<Xt[2][0]) maxZ=Xt[2][0];
        }
	}

	cx=cx/n;
	cy=cy/n;
	cz=cz/n;

	/*uvolneni pameti*/
	free_matrix(Xt);
	free_matrix(A);
	free_matrix(U);
	free_matrix(Ut);
	free_matrix(V);
	free_matrix(b);
	free_matrix(b2);
	free_matrix(y);
	free(w);
}

/**
* Vola prislusne funkce pro vypocet 3D souradnic
*
* @param p matice se shodnymi body
* @param number pocet bodu
* @param K1temp prvni kalibracni matice
* @param K2temp druha kalibracni matice
* @param metric typ modelu: 0-projektivni;1-metricky
* @return   0 - vse probehlo v poradku
*           1 - doslo k chybe
*/
int compute3D(double **p, int number, double **K1temp, double **K2temp, int metric)
{
	double **fm;            //pomocna matice pro ulozeni shodnych bodu
	double **F;             //fundamentali matice
	double **Fn;			//fundamentalni matice normalizovana
	double **P1, **P2;		//projektivni matice
	double **T1,**T2T;		//transformacni matice
	int control;            //prepinac mezi metodami pro vypocet esencialni matice
	double **opt;           //matice s optimalizovanymi souradnicemi

	/*ulozi hodnoty pramateru do globalnich promennych*/
	n=number;

	if(metric==1)
	{
		K1=matrix(3,3);
		K2=matrix(3,3);
		copy_matrix(K1temp,K1,3,3);
		copy_matrix(K2temp,K2,3,3);
	}

	points2D=matrix(n,4);
	copy_matrix(p,points2D,n,4);

	fm=matrix(4,n);			                //pomocna matice na ulozeni shodnych bodu
	transpose(p,fm,n,4);	                //dvojice shodnych bodu jsou ulozeny do jednoho sloupce v poradi x1,y1,x2,y2

	Fn=matrix(3,3);
	F=matrix(3,3);

	control=1;

	if(metric==0)						//projektivni model
    {
        T1=matrix(3,3);					//vytvoreni transformacni matice T1
        T2T=matrix(3,3);				//vytvoreni transponovane transformacni matice T2
        Scaling(fm,T1,T2T);			    //normalizuje souradnice
        Solving_Projective(fm,Fn);      //nalezne fundamentalni matici
        product3(T2T,Fn,T1,F,3,3,3,3);  //denormalizace souradnic

        free_matrix(T1);
        free_matrix(T2T);
    }
	else									//metricky model
    {
        if (control==0)                     //normalizuji se body, z nich pak spocita esencialni matice
        {
            Solving_Metric(fm,Fn);
            copy_matrix(Fn,F,3,3);
        }
        else
        {
			T1=matrix(3,3);					//vytvoreni transformacni matice T1
			T2T=matrix(3,3);				//vytvoreni transponovane transformacni matice T2
			Scaling(fm,T1,T2T);			    //normalizuje souradnice
			Solving_Projective(fm,Fn);
			product3(T2T,Fn,T1,F,3,3,3,3);  //denormalizace souradnic

			free_matrix(T1);
			free_matrix(T2T);
        }
    }


	P1=matrix(3,4);
	P2=matrix(3,4);

	P1=matrixUnit(3,4);

	if(metric==0)
		projectiveP2(F,P2);			//vytvori projektivni matici P2
	else
	{
		if (control==0)
			metricP2(F,P2,0);		//vytvori metrickou matici P2
		else
			metricP2(F,P2,1);       //vytvori metrickou matici P2
	}

	opt=matrix(n,4);
	optimal(p,F,opt);               //optimalizuje souradnice

	inhomogen_method(P1,P2,opt);    //nalezne 3D souradnice nehomogenni metodou


    if(points3D[0][0]>0 && points3D[0][1]<0)
	{
        cx=-cx;
        cy=-cy;
        cz=-cz;
        maxZ=-maxZ;
	}
    /* vytvori bodovy model */
    if(X3D==0 && WRL==0)
        write_points(points3D,n,0);
    if(WRL==1)
        write_points(points3D,n,0);
    if(X3D==1)
        write_points(points3D,n,1);

    free_matrix(opt);
    free_matrix(fm);
	free_matrix(Fn);
	free_matrix(F);
	free_matrix(P1);
	free_matrix(P2);

	return 0;
}

