/* Vypocet trojuhelnikove site z dane mnoziny bodu
 * pomoci metody vyuzivajici vnitrnich vlastnosti teto mnoziny
 * Autor: Beata Turonova
 */
#ifndef _SURFACE_H
#define _SURFACE_H

#include <list>
#include <vector>
using namespace std;


#define use_eps FALSE   //povolit nebo zakazat toleranci u pruseciku trojuhelniku
#define eps 0.000001    //mira tolerance

#define sort(sa,sb)	\
{					\
	double sc;		\
	if(sa>sb)		\
	{				\
		sc=sa;		\
		sa=sb;		\
		sb=sc;		\
	}				\
}

/*vektorovy soucin*/
#define crossProduct(aa,bb,cross)		\
{										\
	cross[0]=aa[1]*bb[2]-aa[2]*bb[1];	\
    cross[1]=aa[2]*bb[0]-aa[0]*bb[2];	\
    cross[2]=aa[0]*bb[1]-aa[1]*bb[0];	\
}

#define innerProduct(aa,bb) (aa[0]*bb[0]+aa[1]*bb[1]+aa[2]*bb[2])

#define innerProduct2(aa,bb) (aa.x*bb.x+aa.y*bb.y+aa.z*bb.z)

#define innerProduct4(aa,bb) (aa.x*bb[0]+aa.y*bb[1]+aa.z*bb[2])

/*vlozi trojuhelnik do seznamu tak, aby se tam zadny nenachazel dvakrat*/
#define insertTriangle(testT,t)												\
{																			\
	list<triangle>::iterator it;											\
	int insert=0;															\
	if(testT.empty())														\
	{																		\
		if (   aE.ida==t.ida || aE.ida==t.idb || aE.ida==t.idc				\
			|| aE.idb==t.ida || aE.idb==t.idb || aE.idb==t.idc)				\
			insert=1;														\
	}																		\
	for(it=testT.begin();it!=testT.end();it++)								\
	{																		\
		if(    ((*it).ida==t.ida && (*it).idb==t.idb && (*it).idc==t.idc)	\
			|| ((*it).ida==t.idb && (*it).idb==t.ida && (*it).idc==t.idc)	\
			|| ((*it).ida==t.ida && (*it).idb==t.idc && (*it).idc==t.idb)	\
			|| ((*it).ida==t.idc && (*it).idb==t.idb && (*it).idc==t.ida)	\
			|| ((*it).ida==t.idb && (*it).idb==t.idc && (*it).idc==t.ida)	\
			|| ((*it).ida==t.idc && (*it).idb==t.ida && (*it).idc==t.idb)	\
			|| aE.ida==t.ida || aE.ida==t.idb || aE.ida==t.idc			    \
			|| aE.idb==t.ida || aE.idb==t.idb || aE.idb==t.idc  			\
			)																\
		{																	\
			insert=1;														\
			break;															\
		}																	\
	}																		\
	if(insert==0)															\
		testT.push_back(t);													\
}

/*otestuje, na ktere strane od roviny dane bodem Pts a normalou normalts lezi
bod dany identifikatorem its; vysledek ulozi do testts*/
#define testSpace2(normalts,Pts,its,testts)	\
{											\
	double D;								\
	double u[3];							\
											\
	u[0]=X[its][0]-Pts.x;					\
	u[1]=X[its][1]-Pts.y;					\
	u[2]=X[its][2]-Pts.z;					\
											\
	D=innerProduct(u,normalts);				\
											\
	if (D<0.0)								\
		testts=-1;							\
	else if (D>0.0)							\
		testts=1;							\
	else									\
		testts=0;							\
}

/*ulozi do d vydalenost bodu A od bodu B*/
#define distance(A,B,d)								\
{													\
	double a,b,c;									\
	double o,p,q;									\
													\
	a=X[A][0];										\
	b=X[A][1];										\
	c=X[A][2];										\
													\
	o=X[B][0];										\
	p=X[B][1];										\
	q=X[B][2];										\
													\
	d=((a-o)*(a-o) + (b-p)*(b-p) + (c-q)*(c-q));	\
}

/*ulozi do s velikost vektoru daneho identifikatory aa a bb*/
#define sizeVector(aa,bb,s)			\
{									\
	double v1,v2,v3;				\
									\
	v1=X[bb][0]-X[aa][0];			\
	v2=X[bb][1]-X[aa][1];			\
	v3=X[bb][2]-X[aa][2];			\
									\
	s=sqrt(v1*v1+v2*v2+v3*v3);		\
}

#define innerProduct3(aa,bb,uu,vv,p)							\
{																\
	double vec1[3], vec2[3];									\
																\
	vec1[0]=X[bb][0]-X[aa][0];									\
	vec1[1]=X[bb][1]-X[aa][1];									\
	vec1[2]=X[bb][2]-X[aa][2];									\
																\
	vec2[0]=X[vv][0]-X[uu][0];									\
	vec2[1]=X[vv][1]-X[uu][1];									\
	vec2[2]=X[vv][2]-X[uu][2];									\
																\
	p=sqrt(vec1[0]*vec2[0]+vec1[1]*vec2[1]+vec1[2]*vec2[2]);	\
}

#define Sub(aa,bb,cc)	\
{						\
	cc.x = aa.x - bb.x;	\
	cc.y = aa.y - bb.y;	\
	cc.z = aa.z - bb.z;	\
}

#define Sub2(dest,v1,v2)	\
{							\
	dest[0]=v1[0]-v2[0];	\
    dest[1]=v1[1]-v2[1];	\
    dest[2]=v1[2]-v2[2];	\
}

/*bod*/
typedef struct xyz{
   double x,y,z;            //souradnice bodu
} xyz;

/*hrana*/
typedef struct edge{
	int ida, idb;			// identifiaktory vrcholu
	int flag;				// 0 aktivni, 1 hranicni
	double center[3];		// souradnice stredu hrany
	double normal[3];		// normala
	double P[3];			// souradnice teziste
	int o;					// protejsi vrchol
}edge;

/*trojuhelnik*/
typedef struct triangle{
	int ida, idb,idc;       //identifikatory bodu
	xyz a, b, c;            //souradnice vrcholu
}triangle;

typedef struct wml{
   double dist;             //vzdalenost
   int id;                  //identifikator bodu
} wml;

/*seznam trojuhelniku*/
typedef struct listT{
	list<triangle> t;
}listT;

/*stena*/
typedef struct face{
	xyz P;                  //bod udavajici stenu
	int test;               //znamenko udavajici, na jake strane steny mame hledat body
	double normal[3];       //normala steny
} face;

/*valec*/
typedef struct cylinder{
	xyz 	cc1, cc2;       //stredy podstav valce
	double	radius;         //polomer
} cylinder;


extern list<triangle> meshT;   //trojuhelnikova sit

/**
* Tridici funkce pro quicksort
*
* Tridi vzdalenosti vzestupne
*
* @param v1 prvni porovnavana vzdalenost
* @param v2 druha porovnavana vzdalenost
*/
int wmlCompare(const void *v1,const void *v2);

/**
* Normalizace vektoru
*
* Spocte velikost daneho vektoru a vydeli touto
* hodnoutou jeho jednotlive slozky
*
* @param vector vektor, ktery ma byt normalizovan
*/
void normVector(double *vector);

/**
* Vytvori stenu
*
* @param P bod udavajici stenu
* @param test udava, ktery poloprostor ma byt testovan na pritomnost bodu
* @param normal normala udavajici rovinu
* @return newF nova stena
*/
face createFace(xyz P, int test, double *normal);

/**
* Vytvori novou hranu
*
* Pro hranu spocita jeji stred, teziste a normalu
* trojuhelniku daneho body ida, idb, a o
*
* @param ida identifikator prvniho bodu udavajiciho hranu
* @param idb identifikator druheho bodu udavajiciho hranu
* @param o identifikator protejsiho bodu hrany
* @param flag status hrany - aktivni nebo hranicni
* @return newE nova hrana
*/
edge createEdge(int ida, int idb, int o, int flag);

/**
* Spocte vzdalenost bodu A od bodu B
*
* @param A identifikator prvniho bodu
* @param B identifikator druheho bodu
* @return d druha mocnina vzdalenosti
*/
double distance2(int A, int B);

/**
* Urci projekci bodu pt
* na primku prochazejici stredy podstav valce cyl
*
* @param cyl valec
* @param pt bod, ktery testujeme
* @return projekce bodu pt na primku
*/
double projection(cylinder cyl, xyz pt);


/**
* Urci bod na primce prochazejici stredy podstav valce cyl
*
* @param cyl valec
* @param t parametr primky
* @return pt bod lezici na dane primce
*/
xyz pointCL(cylinder cyl, double t);

/**
* Urci vzdalenost bodu od primky prchazejici stredy podstav valce
*
* @param cyl valec
* @param t parametr primky
* @param pt bod,jehoz vydalenost od primky chceme zjistit
* @return vzdalenost bodu od primky
*/
double distanceCL(cylinder cyl, double tmin, xyz pt);

/**
* Spocte obsah trojuhelniku daneho body a, b,c
*
* Obsah je roven jedne polovine z odmocniny
* souctu tri determinantu o velikostech 3x3,
* jejichz sloupce jsou tvoreny souradnicemi
* bodu
*
* @param a identifikator prvniho bodu
* @param b identifikator druheho bodu
* @param c identifikator tretiho bodu
* @return d obsah trojuhelniku
*/
double areaTriangle(int a, int b, int c);

/**
* Aktualizuje tabulky MinMax a pointTable
*
* Po pridani noveho trojuhelniku upravi hodnoty
* v tabulce MinMax, popr. upravi statusy bodu a,b
* v pointTable
*
* @param a identifikator prvniho bodu hrany
* @param b identifikator druheho bodu hrany
* @param skip 0, pokud hrana nebyla zpracovana
*             1, pokud byla
*/
void changeTables(int a, int b, int skip);

/**
* Vytvori novy trojuhelnik dany body a, b, c
*
* @param a identifikator prvniho bodu
* @param b identifikator druheho bodu
* @param c identifikator tretiho bodu
* @return triangle novy trojuhelnik
*/
triangle createTriangle(int a, int b, int c);

/**
* Nalezne trojuhelnikovou sit
*
* Pouzit algoritmus vyuzivajici vnitrnich
* vlasntosti vstupni mnoziny bodu
*
* @param points matice se souradnicema 3D bodu
* @param n pocet bodu
* @return   0 pokud vypocet probehl v poradku
*           1 pokud byl vypocet ukoncen kvuli chybe
*/
int mesh(double **points, int n);

#endif
