/* Triangle-triangle intersection test routine,
 * by Tomas Moller, 1997.
 * See article "A Fast Triangle-Triangle Intersection Test",
 * Journal of Graphics Tools, 2(2), 1997
 *
 * Updated June 1999: removed the divisions -- a little faster now!
 *
 * check: http://www.cs.lth.se/home/Tomas_Akenine_Moller/code/
 *
 * Macro testEdge1 is based on Franlin Antonio's gem:
 * "Faster Line Segment Intersection", in Graphics Gems III,
 * pp. 199-202
 */
#ifndef _TRIANGLE_H
#define _TRIANGLE_H


#define testEdge1(V0,U0,U1)									\
	Bx=U0[i0]-U1[i0];										\
  	By=U0[i1]-U1[i1];										\
  	Cx=V0[i0]-U0[i0];										\
  	Cy=V0[i1]-U0[i1];										\
  	f=Ay*Bx-Ax*By;											\
  	d=By*Cx-Bx*Cy;											\
  	if((f>0 && d>=0 && d<=f) || (f<0 && d<=0 && d>=f))		\
  	{														\
    	e=Ax*Cy-Ay*Cx;										\
    	if(f>0)												\
    	{													\
      	if(e>=0 && e<=f) return 1;							\
    	}													\
    	else												\
    	{													\
      	if(e<=0 && e>=f) return 1;							\
    	}													\
}

#define testEdge2(V0,V1,U0,U1,U2)			\
{											\
	double Ax,Ay,Bx,By,Cx,Cy,e,d,f;         \
	Ax=V1[i0]-V0[i0];						\
	Ay=V1[i1]-V0[i1];						\
	/* test hrany U0,U1 s V0,V1 */			\
	testEdge1(V0,U0,U1);					\
	/* test hrany U1,U2 s V0,V1 */			\
	testEdge1(V0,U1,U2);					\
	/* test hrany U2,U1 s V0,V1 */			\
	testEdge1(V0,U2,U0);					\
}

#define testPoint(V0,U0,U1,U2)					\
{												\
	double a,b,c,d0,d1,d2;						\
	/* testuje, jestli T2 nebobsahuje T1 */		\
	/* test,jestli U0,U1,U2 nebobsahuje V0 */	\
	a=U1[i1]-U0[i1];							\
	b=-(U1[i0]-U0[i0]);							\
	c=-a*U0[i0]-b*U0[i1];						\
	d0=a*V0[i0]+b*V0[i1]+c;						\
												\
	a=U2[i1]-U1[i1];							\
	b=-(U2[i0]-U1[i0]);							\
	c=-a*U1[i0]-b*U1[i1];						\
	d1=a*V0[i0]+b*V0[i1]+c;						\
												\
	a=U0[i1]-U2[i1];							\
	b=-(U0[i0]-U2[i0]);							\
	c=-a*U2[i0]-b*U2[i1];						\
	d2=a*V0[i0]+b*V0[i1]+c;						\
	if(d0*d1>0.0)								\
	{											\
		if(d0*d2>0.0) return 1;                 \
	}											\
}

int coplanar_tri_tri(double N[3],double V0[3],double V1[3],double V2[3],
                     double U0[3],double U1[3],double U2[3])
{
	double A[3];
	int i0,i1;

	A[0]=fabs(N[0]);
	A[1]=fabs(N[1]);
	A[2]=fabs(N[2]);
	if(A[0]>A[1])
	{
		if(A[0]>A[2])
		{
			i0=1;
			i1=2;
		}
		else
		{
			i0=0;
			i1=1;
		}
	}
	else
	{
		if(A[2]>A[1])
		{
			i0=0;
			i1=1;
		}
		else
		{
			i0=0;
			i1=2;
		}
	}

	/* test vsech hran trojuhelniku 1 s hranami trojuhelniku 2 */
	testEdge2(V0,V1,U0,U1,U2);
    testEdge2(V1,V2,U0,U1,U2);
    testEdge2(V2,V0,U0,U1,U2);

    /* testuje, jestli neni T1 obsazen cely v T2 nebo naopak */
    testPoint(V0,U0,U1,U2);
    testPoint(U0,V0,V1,V2);

    return 0;
}

#define interval(VV0,VV1,VV2,D0,D1,D2,D0D1,D0D2,A,B,C,X0,X1)			\
{																		\
	if(D0D1>0.0f)														\
		{A=VV2; B=(VV0-VV2)*D2; C=(VV1-VV2)*D2; X0=D2-D0; X1=D2-D1;}	\
	else if(D0D2>0.0f)													\
		{A=VV1; B=(VV0-VV1)*D1; C=(VV2-VV1)*D1; X0=D1-D0; X1=D1-D2;}	\
	else if(D1*D2>0.0f || D0!=0.0f)										\
		{A=VV0; B=(VV1-VV0)*D0; C=(VV2-VV0)*D0; X0=D0-D1; X1=D0-D2;}	\
	else if(D1!=0.0f)													\
		{A=VV1; B=(VV0-VV1)*D1; C=(VV2-VV1)*D1; X0=D1-D0; X1=D1-D2;}	\
	else if(D2!=0.0f)													\
		{A=VV2; B=(VV0-VV2)*D2; C=(VV1-VV2)*D2; X0=D2-D0; X1=D2-D1;}	\
	else																\
		return coplanar_tri_tri(N1,V0,V1,V2,U0,U1,U2);					\
}


/**
* Zjisti, zda se dva trojuhelniky protinaji
*
* @param t1 prvni trojuhelnik
* @param t2 druhy trojuhelnik
* @return   0 - prusecik neexistuje
*           1 - protinaji se
*/
int triangleTest(triangle t1, triangle t2)
{
	double E1[3],E2[3];
	double N1[3],N2[3],d1,d2;
	double du0,du1,du2,dv0,dv1,dv2;
	double D[3];
	double isect1[2], isect2[2];
	double du0du1,du0du2,dv0dv1,dv0dv2;
	int index;
	double vp0,vp1,vp2;
	double up0,up1,up2;
	double bb,cc,max;
	double V0[3],V1[3],V2[3];
	double U0[3],U1[3],U2[3];
	double a1,b1,c1,x0,x1;
	double d,e,f,y0,y1;
	double xx,yy,xxyy,tmp;

	//if(    t1.ida==t2.ida || t1.ida==t2.idb || t1.ida==t2.idc
	//	|| t1.idb==t2.ida || t1.idb==t2.idb || t1.idb==t2.idc
	//	|| t1.idc==t2.ida || t1.idc==t2.idb || t1.idc==t2.idc
	//	)
	//	return 1;

	V0[0]=t1.a.x;
	V0[1]=t1.a.y;
	V0[2]=t1.a.z;

	V1[0]=t1.b.x;
	V1[1]=t1.b.y;
	V1[2]=t1.b.z;

	V2[0]=t1.c.x;
	V2[1]=t1.c.y;
	V2[2]=t1.c.z;

	U0[0]=t2.a.x;
	U0[1]=t2.a.y;
	U0[2]=t2.a.z;

	U1[0]=t2.b.x;
	U1[1]=t2.b.y;
	U1[2]=t2.b.z;

	U2[0]=t2.c.x;
	U2[1]=t2.c.y;
	U2[2]=t2.c.z;

	/* spocita rovnici roviny trojuhelniku V0,V1,V2 */
	Sub2(E1,V1,V0);
	Sub2(E2,V2,V0);
	crossProduct(E1,E2,N1);
	d1=-innerProduct(N1,V0);
	/* rovnice roviny 1: N1.X+d1=0 */

	/* spocita vzdalenosti U0,U1 a U2 od roviny 1 */
	du0=innerProduct(N1,U0)+d1;
	du1=innerProduct(N1,U1)+d1;
	du2=innerProduct(N1,U2)+d1;

	#if use_eps==TRUE
		if(fabs(du0)<eps) du0=0.0;
		if(fabs(du1)<eps) du1=0.0;
		if(fabs(du2)<eps) du2=0.0;
	#endif

	du0du1=du0*du1;
	du0du2=du0*du2;

	if(du0du1>0.0f && du0du2>0.0f)		/* nenulove se stejnym znamenkem => */
		return 0;						/* =>trojuhelniky se neprotinaji */

	/* spocita rovnici roviny trojuhelniku U0,U1,U2 */
	Sub2(E1,U1,U0);
	Sub2(E2,U2,U0);
	crossProduct(E1,E2,N2);
	d2=-innerProduct(N2,U0);
	/* rovnice roviny 2: N2.X+d2=0 */

	/*  spocita vzdalenosti U0,U1 a U2 od roviny 2 */
	dv0=innerProduct(N2,V0)+d2;
	dv1=innerProduct(N2,V1)+d2;
	dv2=innerProduct(N2,V2)+d2;

	#if use_eps==TRUE
		if(fabs(dv0)<eps) dv0=0.0;
		if(fabs(dv1)<eps) dv1=0.0;
		if(fabs(dv2)<eps) dv2=0.0;
	#endif

	dv0dv1=dv0*dv1;
	dv0dv2=dv0*dv2;

	if(dv0dv1>0.0f && dv0dv2>0.0f)		/* nenulove se stejnym znamenkem => */
		return 0;						/* =>trojuhelniky se neprotinaji */

	/* smer prusecnice */
	crossProduct(N1,N2,D);

	/* spocita a oznaci nejvetsi cast D */
	max=fabs(D[0]);
	index=0;
	bb=fabs(D[1]);
	cc=fabs(D[2]);
	if(bb>max) max=bb,index=1;
	if(cc>max) max=cc,index=2;

	/* projekce na L*/
	vp0=V0[index];
	vp1=V1[index];
	vp2=V2[index];

	up0=U0[index];
	up1=U1[index];
	up2=U2[index];

	/* spocita interval pro trojuhelnik 1 */
	interval(vp0,vp1,vp2,dv0,dv1,dv2,dv0dv1,dv0dv2,a1,b1,c1,x0,x1);

	/* spocita interval pro trojuhelnik 2 */
	interval(up0,up1,up2,du0,du1,du2,du0du1,du0du2,d,e,f,y0,y1);


	xx=x0*x1;
	yy=y0*y1;
	xxyy=xx*yy;

	tmp=a1*xxyy;
	isect1[0]=tmp+b1*x1*yy;
	isect1[1]=tmp+c1*x0*yy;

	tmp=d*xxyy;
	isect2[0]=tmp+e*xx*y1;
	isect2[1]=tmp+f*xx*y0;

	sort(isect1[0],isect1[1]);
	sort(isect2[0],isect2[1]);

	if(isect1[1]<isect2[0] || isect2[1]<isect1[0])
		return 0;
	else
		return 1;
}

#endif
